/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __lr10_dev_nxbar_tc_global_ip_h__
#define __lr10_dev_nxbar_tc_global_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST                                                                   0x00000240     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST                                                                   0x00000440     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST                                                                   0x00000640     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST                                                                   0x00000840     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST                                                                   0x00000a40     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST                                                                   0x00000c40     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST                                                                   0x00000e40     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST                                                                   0x00001040     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST                                                                   0x00001240     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_BUFFER_OVERFLOW                                           0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_BUFFER_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW                                          1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_BUFFER_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_EGRESS_CREDIT_OVERFLOW                                            2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_EGRESS_CREDIT_OVERFLOW_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW                                           3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_EGRESS_CREDIT_UNDERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_NON_BURSTY_PKT                                            4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_NON_BURSTY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_NON_STICKY_PKT                                            5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_NON_STICKY_PKT_INIT                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC                                        6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_INGRESS_BURST_GT_9_DATA_VC_INIT                                   0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_EGRESS_CDT_PARITY_ERROR                                           7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_EGRESS_CDT_PARITY_ERROR_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_VC                                                            15:13          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_VC_INIT                                                       0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_DST                                                           19:16          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_DST_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_SRC                                                           25:24          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_SRC_INIT                                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_TILE_SRC                                                      31:28          /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FIRST_LOG_TILE_SRC_INIT                                                 0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN                                                           0x0000023c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN                                                           0x0000043c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN                                                           0x0000063c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN                                                           0x0000083c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN                                                           0x00000a3c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN                                                           0x00000c3c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN                                                           0x00000e3c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN                                                           0x0000103c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN                                                           0x0000123c     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW                                   0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BUFFER_OVERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW                                  1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW_INIT                             0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BUFFER_UNDERFLOW__PROD                            0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW                                    2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CREDIT_OVERFLOW__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW                                   3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CREDIT_UNDERFLOW__PROD                             0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT                                    4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_NON_BURSTY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT                                    5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT_INIT                               0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_NON_STICKY_PKT__PROD                              0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC                                6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC_INIT                           0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_INGRESS_BURST_GT_9_DATA_VC__PROD                          0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR                                   7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR_INIT                              0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_FATAL_INTR_EN_EGRESS_CDT_PARITY_ERROR__PROD                             0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS                                                                  0x00000234     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS                                                                  0x00000434     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS                                                                  0x00000634     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS                                                                  0x00000834     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS                                                                  0x00000a34     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS                                                                  0x00000c34     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS                                                                  0x00000e34     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS                                                                  0x00001034     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS                                                                  0x00001234     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_BUFFER_OVERFLOW                                          0:0            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_BUFFER_OVERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW                                         1:1            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_BUFFER_UNDERFLOW_INIT                                    0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_EGRESS_CREDIT_OVERFLOW                                           2:2            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_EGRESS_CREDIT_OVERFLOW_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW                                          3:3            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_EGRESS_CREDIT_UNDERFLOW_INIT                                     0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_NON_BURSTY_PKT                                           4:4            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_NON_BURSTY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_NON_STICKY_PKT                                           5:5            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_NON_STICKY_PKT_INIT                                      0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC                                       6:6            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_INGRESS_BURST_GT_9_DATA_VC_INIT                                  0x00000000     /* RWI-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_EGRESS_CDT_PARITY_ERROR                                          7:7            /* RWIVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_STATUS_EGRESS_CDT_PARITY_ERROR_INIT                                     0x00000000     /* RWI-V */

#define NV_NXBAR_TC_TILEOUT0_ERR_CYA                                                                     0x00000230     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT0_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT0_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT0_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT0_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT1_ERR_CYA                                                                     0x00000430     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT1_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT1_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT1_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT1_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT2_ERR_CYA                                                                     0x00000630     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT2_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT2_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT2_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT2_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT3_ERR_CYA                                                                     0x00000830     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT3_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT3_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT3_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT3_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT4_ERR_CYA                                                                     0x00000a30     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT4_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT4_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT4_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT4_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT5_ERR_CYA                                                                     0x00000c30     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT5_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT5_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT5_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT5_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT6_ERR_CYA                                                                     0x00000e30     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT6_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT6_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT6_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT6_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT7_ERR_CYA                                                                     0x00001030     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT7_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT7_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT7_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT7_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_TILEOUT8_ERR_CYA                                                                     0x00001230     /* RW-4R */
#define NV_NXBAR_TC_TILEOUT8_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL                                         0:0            /* RWEVF */
#define NV_NXBAR_TC_TILEOUT8_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_ENABLE                                  0x00000001     /* RW--V */
#define NV_NXBAR_TC_TILEOUT8_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL_DISABLE                                 0x00000000     /* RWE-V */
#define NV_NXBAR_TC_TILEOUT8_ERR_CYA_SRCID_UPDATE_AT_EGRESS_CTRL__PROD                                   0x00000001     /* RW--V */

#define NV_NXBAR_TC_ERROR_STATUS                                                                         0x00000090     /* R--4R */
#define NV_NXBAR_TC_ERROR_STATUS_TILE0                                                                   0:0            /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILE0_DEFAULT                                                           0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILE1                                                                   1:1            /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILE1_DEFAULT                                                           0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILE2                                                                   2:2            /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILE2_DEFAULT                                                           0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILE3                                                                   3:3            /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILE3_DEFAULT                                                           0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT0                                                                16:16          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT0_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT1                                                                17:17          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT1_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT2                                                                18:18          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT2_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT3                                                                19:19          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT3_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT4                                                                20:20          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT4_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT5                                                                21:21          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT5_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT6                                                                22:22          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT6_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT7                                                                23:23          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT7_DEFAULT                                                        0x00000000     /* R---V */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT8                                                                24:24          /* R--VF */
#define NV_NXBAR_TC_ERROR_STATUS_TILEOUT8_DEFAULT                                                        0x00000000     /* R---V */

#define NV_NXBAR_TC_PRI_NXBAR_TC_CG                          0x00000048 /* RWE4R */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_IDLE_CG_DLY_CNT                 5:0 /* RWEVF */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_IDLE_CG_DLY_CNT_HWINIT   0x00000000 /* RWE-V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_IDLE_CG_DLY_CNT__PROD    0x00000004 /* RW--V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_IDLE_CG_EN                      6:6 /* RWEVF */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_IDLE_CG_EN_ENABLED       0x00000001 /* RW--V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_IDLE_CG_EN_DISABLED      0x00000000 /* RWE-V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_IDLE_CG_EN__PROD         0x00000001 /* RW--V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STATE_CG_EN                     7:7 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STATE_CG_EN_ENABLED      0x00000001 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STATE_CG_EN_DISABLED     0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STATE_CG_EN__PROD        0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STALL_CG_DLY_CNT               13:8 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STALL_CG_DLY_CNT_HWINIT  0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STALL_CG_DLY_CNT__PROD   0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STALL_CG_EN                   14:14 /* RWEVF */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STALL_CG_EN_ENABLED      0x00000001 /* RW--V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STALL_CG_EN_DISABLED     0x00000000 /* RWE-V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_STALL_CG_EN__PROD        0x00000000 /* RW--V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_QUIESCENT_CG_EN               15:15 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_QUIESCENT_CG_EN_ENABLED  0x00000001 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_QUIESCENT_CG_EN_DISABLED 0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_QUIESCENT_CG_EN__PROD    0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_WAKEUP_DLY_CNT                19:16 /* RWEVF */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_WAKEUP_DLY_CNT_HWINIT    0x00000000 /* RWE-V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_WAKEUP_DLY_CNT__PROD     0x00000000 /* RW--V */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_CNT                 23:20 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_CNT_FULLSPEED  0x0000000f /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_CNT__PROD      0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_DI_DT_SKEW_VAL                27:24 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_DI_DT_SKEW_VAL_HWINIT    0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_DI_DT_SKEW_VAL__PROD     0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_EN                  28:28 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_EN_ENABLED     0x00000001 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_EN_DISABLED    0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_EN__PROD       0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_SW_OVER             29:29 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_SW_OVER_EN     0x00000001 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_SW_OVER_DIS    0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_THROT_CLK_SW_OVER__PROD  0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_PAUSE_CG_EN                   30:30 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_PAUSE_CG_EN_ENABLED      0x00000001 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_PAUSE_CG_EN_DISABLED     0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_PAUSE_CG_EN__PROD        0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_HALT_CG_EN                    31:31 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_HALT_CG_EN_ENABLED       0x00000001 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_HALT_CG_EN_DISABLED      0x00000000 /*       */
#define NV_NXBAR_TC_PRI_NXBAR_TC_CG_HALT_CG_EN__PROD         0x00000000 /*       */
#endif // __lr10_dev_nxbar_tc_global_ip_h__
