// This file is automatically generated by rmconfig - DO NOT EDIT!
//
// Profile:  shipping-gpus-openrm
// Template: templates/gt_eng_empty.h
//
// The file is added to smoonth NVOC migration.  After converting a module to
// NVOC class, the stale generated headers in output directory causes failures
// of incremental builds.  This file ensures the content of the old header is
// removed.
//
