/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2016 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __clc37dcrcnotif_h__
#define __clc37dcrcnotif_h__
/* This file is autogenerated.  Do not edit */

#define NVC37D_NOTIFIER_CRC_STATUS_0                                           0x00000000
#define NVC37D_NOTIFIER_CRC_STATUS_0_DONE                                      0:0
#define NVC37D_NOTIFIER_CRC_STATUS_0_DONE_FALSE                                0x00000000
#define NVC37D_NOTIFIER_CRC_STATUS_0_DONE_TRUE                                 0x00000001
#define NVC37D_NOTIFIER_CRC_STATUS_0_COMPOSITOR_OVERFLOW                       3:3
#define NVC37D_NOTIFIER_CRC_STATUS_0_COMPOSITOR_OVERFLOW_FALSE                 0x00000000
#define NVC37D_NOTIFIER_CRC_STATUS_0_COMPOSITOR_OVERFLOW_TRUE                  0x00000001
#define NVC37D_NOTIFIER_CRC_STATUS_0_RG_OVERFLOW                               4:4
#define NVC37D_NOTIFIER_CRC_STATUS_0_RG_OVERFLOW_FALSE                         0x00000000
#define NVC37D_NOTIFIER_CRC_STATUS_0_RG_OVERFLOW_TRUE                          0x00000001
#define NVC37D_NOTIFIER_CRC_STATUS_0_PRIMARY_OUTPUT_OVERFLOW                   5:5
#define NVC37D_NOTIFIER_CRC_STATUS_0_PRIMARY_OUTPUT_OVERFLOW_FALSE             0x00000000
#define NVC37D_NOTIFIER_CRC_STATUS_0_PRIMARY_OUTPUT_OVERFLOW_TRUE              0x00000001
#define NVC37D_NOTIFIER_CRC_STATUS_0_COUNT                                     27:16
#define NVC37D_NOTIFIER_CRC_CRC_ENTRY0_11                                      0x0000000B
#define NVC37D_NOTIFIER_CRC_CRC_ENTRY0_11_COMPOSITOR_CRC                       31:0
#define NVC37D_NOTIFIER_CRC_CRC_ENTRY0_12                                      0x0000000C
#define NVC37D_NOTIFIER_CRC_CRC_ENTRY0_12_RG_CRC                               31:0
#define NVC37D_NOTIFIER_CRC_CRC_ENTRY0_13                                      0x0000000D
#define NVC37D_NOTIFIER_CRC_CRC_ENTRY0_13_PRIMARY_OUTPUT_CRC                   31:0
#define NVC37D_NOTIFIER_CRC_CRC_ENTRY1_21                                      0x00000015

#endif // __clc37dcrcnotif_h__
