// Generated by the protocol buffer compiler.  DO NOT EDIT!

#include "nvtypes.h"
#include "prbrt.h"
#include "g_gr_pb.h"

// 'GR_ERR_TYP' enum descriptor
static const PRB_ENUM_MAPPING prb_enum_mappings_gr_err_typ[] = {
    {
        255,
        PRB_MAYBE_ENUM_NAME("GR_ERR_TYP_UNKNOWN")
    },
};

const PRB_ENUM_DESC prb_enums_gr_gr_err_typ = {
    prb_enum_mappings_gr_err_typ,
    1,
   PRB_MAYBE_ENUM_NAME("GR_ERR_TYP")
};

// 'GR_EXCPTN_SUBTYP' enum descriptor
static const PRB_ENUM_MAPPING prb_enum_mappings_gr_excptn_subtyp[] = {
    {
        255,
        PRB_MAYBE_ENUM_NAME("GR_EXCPTN_SUBTYP_UNKNOWN")
    },
};

const PRB_ENUM_DESC prb_enums_gr_gr_excptn_subtyp = {
    prb_enum_mappings_gr_excptn_subtyp,
    1,
   PRB_MAYBE_ENUM_NAME("GR_EXCPTN_SUBTYP")
};

// 'Exception' field defaults

// 'Exception' field descriptors
const PRB_FIELD_DESC prb_fields_gr_exception[] = {
    {
        1,
        {
            PRB_REQUIRED,
            PRB_MESSAGE,
            0,
        },
        GR_EXCEPTION_EXCEPTIONDATA,
        0,
        PRB_MAYBE_FIELD_NAME("ed")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        REGS_REGSANDMEM,
        0,
        PRB_MAYBE_FIELD_NAME("nv50_regs")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        REGS_REGSANDMEM,
        0,
        PRB_MAYBE_FIELD_NAME("gt200_regs")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        REGS_REGSANDMEM,
        0,
        PRB_MAYBE_FIELD_NAME("gt212_regs")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        5,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        REGS_REGSANDMEM,
        0,
        PRB_MAYBE_FIELD_NAME("gf100_regs")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'ExceptionData' field defaults

// 'ExceptionData' field descriptors
const PRB_FIELD_DESC prb_fields_gr_exception_exceptiondata[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("gpu_instance")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("ch_id")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("error_sequence_number")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_OPTIONAL,
            PRB_UINT64,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("time_stamp")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// Message descriptors
const PRB_MSG_DESC prb_messages_gr[] = {
    {
        5,
        prb_fields_gr_exception,
        PRB_MAYBE_MESSAGE_NAME("Gr.Exception")
    },
    {
        4,
        prb_fields_gr_exception_exceptiondata,
        PRB_MAYBE_MESSAGE_NAME("Gr.Exception.ExceptionData")
    },
};

// Service descriptors
const PRB_SERVICE_DESC prb_services_gr[] = {
   { 0 }
};

