/*
 * SPDX-FileCopyrightText: Copyright (c) 2016-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


/* THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT! */



#if defined(BINDATA_INCLUDE_DATA)
//
// FUNCTION: spdmGetBinArchiveIndividualL3Certificate_GB207("BINDATA_LABEL_CERTIFICATE_PEM")
// FILE NAME: inc/kernel/spdm/bin/blackwell/gb207_l3_cert.pem.bin
// FILE TYPE: BINARY
// VAR NAME: N/A
// COMPRESSION: YES
// COMPLEX_STRUCT: NO
// DATA SIZE (bytes): 989
// COMPRESSED SIZE (bytes): 748
// EXTERNAL: NO
//
static BINDATA_CONST NV_DECLARE_ALIGNED(NvU8, 8) spdmBinArchiveIndividualL3Certificate_GB207_BINDATA_LABEL_CERTIFICATE_PEM_data[] = 
{
    0x65, 0x53, 0xcb, 0x92, 0xa2, 0x4a, 0x10, 0xdd, 0xf3, 0x15, 0x77, 0x6f, 0x74, 0x48, 0xa3, 0x32,
    0xba, 0x98, 0x45, 0x26, 0x55, 0x14, 0x85, 0x16, 0xc8, 0x43, 0x10, 0x77, 0x02, 0x8a, 0x0d, 0x08,
    0x3e, 0x29, 0xe5, 0xeb, 0xaf, 0x76, 0xc7, 0x9d, 0x59, 0xdc, 0x8c, 0x38, 0x9b, 0x8c, 0xcc, 0x93,
    0xaf, 0x93, 0x1f, 0x1f, 0x2f, 0x43, 0xca, 0xb8, 0xf3, 0x8f, 0x41, 0xfd, 0x90, 0x9b, 0xdc, 0x80,
    0x90, 0xbe, 0x9d, 0x1f, 0x8a, 0xe0, 0xdc, 0xb8, 0x10, 0xc3, 0x80, 0xd2, 0x2d, 0x40, 0x72, 0x84,
    0x82, 0x7b, 0x81, 0x1e, 0x85, 0xba, 0x36, 0x93, 0xb6, 0xa1, 0xcf, 0x66, 0xd2, 0xd8, 0xf3, 0x71,
    0x92, 0x87, 0x3d, 0xcc, 0xb1, 0x28, 0xce, 0x87, 0xaa, 0x74, 0x97, 0x9e, 0x47, 0xa0, 0x87, 0x99,
    0xf0, 0xc7, 0x52, 0xb1, 0x20, 0x21, 0xd1, 0xcb, 0x41, 0x30, 0x72, 0xa3, 0xaa, 0xa6, 0xc1, 0x8a,
    0x16, 0xbe, 0xca, 0x9f, 0xc2, 0x2c, 0x8a, 0x20, 0xf6, 0xeb, 0xb4, 0xf1, 0x4f, 0xb9, 0x55, 0x3d,
    0x58, 0x0f, 0x1b, 0x2c, 0x9c, 0x08, 0xa1, 0x15, 0xb4, 0x9a, 0xc4, 0xc1, 0xca, 0xb7, 0xbd, 0x00,
    0x89, 0x92, 0x8e, 0x6c, 0xf9, 0xc2, 0x21, 0x67, 0x75, 0x97, 0x96, 0x50, 0x98, 0x52, 0x7d, 0x8a,
    0x1e, 0x34, 0x51, 0x82, 0x14, 0xe4, 0x0d, 0xdc, 0x32, 0x98, 0x4e, 0xdc, 0xb0, 0x9a, 0x88, 0x90,
    0xf7, 0x6f, 0x38, 0x61, 0xf5, 0xe9, 0x46, 0xad, 0x8c, 0x7a, 0x7a, 0x51, 0x84, 0x51, 0x31, 0xf8,
    0x5c, 0x51, 0x90, 0xf2, 0x2b, 0xac, 0x37, 0xb6, 0x4f, 0x6b, 0xe4, 0x34, 0x37, 0x5e, 0x04, 0x23,
    0x6e, 0xe2, 0x33, 0x1d, 0x6d, 0x4e, 0x99, 0xf6, 0x22, 0x3f, 0x46, 0x4f, 0x4e, 0x6b, 0xe2, 0x05,
    0xff, 0x11, 0xd3, 0x54, 0x60, 0xc5, 0x94, 0x77, 0xb2, 0x51, 0xc8, 0xe0, 0x6f, 0xb2, 0xd3, 0x65,
    0x0d, 0x76, 0xd9, 0xd1, 0x54, 0xb7, 0xf1, 0xec, 0x2e, 0xbc, 0xab, 0x34, 0xbc, 0xef, 0x31, 0x19,
    0x95, 0x76, 0xb4, 0xea, 0x51, 0x13, 0x08, 0x0c, 0x9f, 0x67, 0x16, 0x88, 0xf1, 0x4c, 0x81, 0x82,
    0x32, 0x0c, 0xee, 0x88, 0x00, 0x5f, 0xa0, 0x71, 0x40, 0x63, 0x76, 0xc1, 0xda, 0x5c, 0x2e, 0xe5,
    0xf6, 0xd0, 0xa9, 0x63, 0xef, 0x3c, 0x1c, 0xdf, 0xe8, 0x20, 0x0e, 0xf0, 0x73, 0x68, 0x3f, 0xd8,
    0xce, 0x65, 0xa4, 0x6f, 0x3f, 0x37, 0xb8, 0xae, 0xac, 0x2a, 0x8e, 0xdd, 0x2a, 0x50, 0x70, 0x7d,
    0xd8, 0xb0, 0x51, 0x7c, 0x83, 0x73, 0xa0, 0xd6, 0xfa, 0xe3, 0x4e, 0x7f, 0xb9, 0xd6, 0x45, 0x6e,
    0x63, 0x2d, 0xcf, 0xfd, 0x87, 0x3f, 0x31, 0xe7, 0xd5, 0xcd, 0x9f, 0x6f, 0xcc, 0x66, 0xf4, 0x39,
    0x18, 0xb4, 0xdc, 0x7c, 0x74, 0xdb, 0xd5, 0xe1, 0xd7, 0x7a, 0x1f, 0x5e, 0x3b, 0x74, 0xf6, 0x61,
    0xc9, 0x95, 0xd2, 0xd1, 0xcb, 0xbc, 0x0d, 0x22, 0xf7, 0x79, 0x71, 0x12, 0x1f, 0x27, 0x1b, 0xdd,
    0x45, 0xad, 0x37, 0x50, 0x23, 0xb0, 0x7c, 0x6f, 0xdd, 0xf2, 0x5f, 0x0d, 0xef, 0xa7, 0x14, 0x43,
    0x20, 0xe0, 0x59, 0x43, 0x01, 0xe3, 0xf7, 0xce, 0x72, 0x22, 0x29, 0x0e, 0xa5, 0x47, 0x95, 0xf7,
    0xed, 0xb1, 0x84, 0xe9, 0x4f, 0xf0, 0x94, 0x3a, 0x21, 0xf4, 0x82, 0xb0, 0x62, 0xa1, 0xc3, 0xed,
    0xf0, 0x75, 0x6b, 0x73, 0xcb, 0x97, 0xee, 0xd7, 0xb4, 0x4b, 0x46, 0xf6, 0x75, 0x71, 0x9c, 0x54,
    0xdb, 0xb5, 0xb8, 0xa7, 0xcd, 0xe6, 0xb4, 0x61, 0xf5, 0x61, 0x71, 0x74, 0x3a, 0x25, 0x0d, 0x66,
    0x65, 0x76, 0x94, 0x5d, 0x4a, 0xf8, 0x6d, 0xa3, 0xbd, 0xee, 0x6f, 0x15, 0xf7, 0x77, 0xb0, 0x20,
    0x19, 0x33, 0x8c, 0x2b, 0x03, 0x6f, 0x65, 0xe2, 0xab, 0x1a, 0x1a, 0x57, 0x39, 0x0f, 0xa1, 0x79,
    0x89, 0xe9, 0x82, 0x05, 0x35, 0xd1, 0xcb, 0x24, 0x24, 0x49, 0xaa, 0x6c, 0x2d, 0x5f, 0xcd, 0x48,
    0xdb, 0x2d, 0xb4, 0x17, 0xd1, 0x08, 0xee, 0xe9, 0xd1, 0x3f, 0x65, 0xcf, 0xc9, 0x3d, 0x3f, 0xd6,
    0xd5, 0x36, 0xa6, 0xf7, 0x44, 0x9b, 0xdd, 0x04, 0xaa, 0x3f, 0x6d, 0x17, 0x5e, 0x8c, 0xe8, 0x0d,
    0x69, 0xb3, 0x30, 0x9a, 0xcb, 0xb9, 0xe5, 0xf1, 0x2c, 0x50, 0x3a, 0x58, 0xed, 0xda, 0xa3, 0xd6,
    0xaf, 0x7d, 0x26, 0x77, 0x3d, 0xec, 0xbf, 0x47, 0x09, 0x04, 0x65, 0x04, 0xe2, 0x02, 0xc3, 0xe1,
    0x22, 0xe3, 0x18, 0x4e, 0xf4, 0xd1, 0x40, 0xa6, 0x97, 0x95, 0x3c, 0xf6, 0x22, 0xea, 0x06, 0xd9,
    0xfe, 0x9c, 0x6a, 0xe1, 0x8f, 0xb4, 0x95, 0x3f, 0xda, 0x96, 0x4e, 0x03, 0x04, 0x2b, 0x28, 0x31,
    0x18, 0x68, 0x93, 0x73, 0x73, 0xc8, 0x4f, 0xad, 0x16, 0x3e, 0x86, 0xde, 0x58, 0x5d, 0xea, 0xe7,
    0xad, 0x3d, 0xb5, 0x9d, 0xc7, 0x94, 0x35, 0x96, 0x90, 0xeb, 0x64, 0x37, 0x5c, 0xf4, 0x64, 0x27,
    0xcf, 0x59, 0x16, 0x28, 0x87, 0x70, 0x39, 0x5f, 0x24, 0x81, 0xb5, 0x86, 0x6b, 0x33, 0x2d, 0xce,
    0x0f, 0x27, 0x4d, 0x0c, 0x81, 0xd7, 0xc4, 0xd6, 0x56, 0xf6, 0x92, 0xa5, 0xfb, 0xba, 0x3e, 0x0d,
    0x47, 0xb1, 0xb7, 0xe0, 0x5a, 0x4c, 0xbf, 0x12, 0xfd, 0xe8, 0x5e, 0x54, 0xfb, 0xa9, 0xd2, 0xc7,
    0x6d, 0xd4, 0x5c, 0xf5, 0x89, 0x51, 0x2b, 0xce, 0x25, 0x91, 0xee, 0x7c, 0xf9, 0x68, 0x23, 0xf5,
    0xc4, 0x87, 0x2b, 0xe2, 0x91, 0xf4, 0x59, 0xc2, 0xef, 0xdf, 0xca, 0xf7, 0xb3, 0x52, 0x87, 0xfc,
    0xff, 0x81, 0xff, 0x05, 0xd2, 0x4a, 0x3c, 0xa8, 0xdd, 0x03, 0x00, 0x00,
};
#endif // defined(BINDATA_INCLUDE_DATA)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)
BINDATA_STORAGE_PVT spdmBinArchiveIndividualL3Certificate_GB207_BINDATA_LABEL_CERTIFICATE_PEM_storage_pvt;
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)
{
    989,                                                               // uncompressed data size (bytes)
    748,                                                               // compressed data size (bytes)
    spdmBinArchiveIndividualL3Certificate_GB207_BINDATA_LABEL_CERTIFICATE_PEM_data, // compressed data pointer
    BINDATA_FLAG_COMPRESSED | BINDATA_FLAG_EXTERNAL_VMWARE,            // u32 flags (see bin_data_pvt.h)
    0,                                                                 // Unused
},
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)


#if defined(BINDATA_INCLUDE_ARCHIVE)
//
// Bindata Archive structure
//
static const BINDATA_ARCHIVE __spdmGetBinArchiveIndividualL3Certificate_GB207 =
{
    1,  // entryNum
    {
        // entries[] : { label, pBinStorage }
        { BINDATA_LABEL_CERTIFICATE_PEM , (const PBINDATA_STORAGE) &g_bindata_pvt[__idx_spdmBinArchiveIndividualL3Certificate_GB207_BINDATA_LABEL_CERTIFICATE_PEM] },
    }
};

#endif // defined(BINDATA_INCLUDE_ARCHIVE)



#if defined(BINDATA_INCLUDE_FUNCTION)
const BINDATA_ARCHIVE *spdmGetBinArchiveIndividualL3Certificate_GB207(struct Spdm *pSpdm)
{
    return &__spdmGetBinArchiveIndividualL3Certificate_GB207;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION)


#if defined(BINDATA_INCLUDE_FUNCTION_STUB)
const BINDATA_ARCHIVE *spdmGetBinArchiveIndividualL3Certificate_GB207(struct Spdm *pSpdm)
{
    return NULL;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION_STUB)





