/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __ls10_dev_therm_h__
#define __ls10_dev_therm_h__
/* This file is autogenerated.  Do not edit */
#define NV_THERM                               0x067fff:0x066000 /* RW--D */
#define NV_THERM_I2CS_SCRATCH                                                  0x000660bc /* RW-4R */
#define NV_THERM_I2CS_SCRATCH_DATA                                                   31:0 /* RWIVF */
#define NV_THERM_I2CS_SCRATCH_DATA_INIT                                        0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_0                                           0x00066f10 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_1                                           0x00066f14 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_2                                           0x00066f18 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_3                                           0x00066f1c /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_4                                           0x00066f20 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_5                                           0x00066f24 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_MODIFICATIONS                 0x00066f28 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_0_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_MODIFICATIONS                 0x00066f2c /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_1_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_MODIFICATIONS                 0x00066f30 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_2_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_MODIFICATIONS                 0x00066f34 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_3_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_MODIFICATIONS                 0x00066f38 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_4_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_MODIFICATIONS                 0x00066f3c /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_5_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_MAXIMUM_TEMPERATURE                                    0x00066f40 /* R--4R */
#define NV_THERM_TSENSE_MAXIMUM_TEMPERATURE_MAXIMUM_TEMPERATURE                      13:0 /* R--VF */
#define NV_THERM_TSENSE_MAXIMUM_TEMPERATURE_MAX_TEMP_SENSE_NUMBER                   24:16 /* R--VF */
#define NV_THERM_TSENSE_THRESHOLD_TEMPERATURES                                 0x00066f44 /* RW-4R */
#define NV_THERM_TSENSE_THRESHOLD_TEMPERATURES_WARNING_TEMPERATURE                   13:0 /* RWIVF */
#define NV_THERM_TSENSE_THRESHOLD_TEMPERATURES_WARNING_TEMPERATURE_INIT        0x00000c80 /* RWI-V */
#define NV_THERM_TSENSE_THRESHOLD_TEMPERATURES_OVERTEMP_TEMPERATURE                 29:16 /* RWIVF */
#define NV_THERM_TSENSE_THRESHOLD_TEMPERATURES_OVERTEMP_TEMPERATURE_INIT       0x00000dc0 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_6                                           0x00066fa0 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_7                                           0x00066fa4 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_8                                           0x00066fa8 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE                                     13:0 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_INIT                          0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_SOURCE                                         16:16 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_SOURCE_INIT                               0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_VALID                              30:30 /* R-IVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_VALID_INIT                    0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_EN                                 31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_EN_INIT                       0x00000001 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_MODIFICATIONS                 0x00066fb0 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_6_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_MODIFICATIONS                 0x00066fb4 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_7_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_MODIFICATIONS                 0x00066fb8 /* RW-4R */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET    13:0 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OFFSET_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE  29:16 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_INIT  0x00000000 /* RWI-V */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN  31:31 /* RWIVF */
#define NV_THERM_TSENSE_U2_A_0_BJT_8_TEMPERATURE_MODIFICATIONS_TEMPERATURE_OVERRIDE_EN_INIT  0x00000000 /* RWI-V */
#endif // __ls10_dev_therm_h__
