/*
 * SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef INTR_COMMON_H
#define INTR_COMMON_H 1
/*! Common defines used by both Intr and OBJINTRABLE */


/*! Kinds of interrupts that a unit can have.
 *
 * Different interrupt vectors can route to the same MC_ENGINE_IDX_* (and its
 * associated #IntrService).
 * This enum is used to disambiguate which handler function within an
 * #IntrService should be called depending on the actual interrupt vector.
 */
typedef enum {
    /*!
     * Legacy concept of "stalling" interrupts.
     *
     * These may have a RETRIGGER mechanism.
     */
    INTR_KIND_INTERRUPT,
    /*! Notification "non-stalling" interrupts. */
    INTR_KIND_NOTIFICATION,
    INTR_KIND_COUNT
} INTR_KIND;


#endif /* ifndef INTR_COMMON_H */
