/*
 * SPDX-FileCopyrightText: Copyright (c) 2016-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


/* THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT! */



#if defined(BINDATA_INCLUDE_DATA)
//
// FUNCTION: kgspGetBinArchiveGspRmBoot_TU102("BINDATA_LABEL_UCODE_IMAGE")
// FILE NAME: kernel/inc/gsprm/bin/g_gsprm_tu10x_riscv_image.bin
// FILE TYPE: BINARY
// VAR NAME: N/A
// COMPRESSION: YES
// COMPLEX_STRUCT: NO
// DATA SIZE (bytes): 4096
// COMPRESSED SIZE (bytes): 811
// EXTERNAL: NO
//
static BINDATA_CONST NV_DECLARE_ALIGNED(NvU8, 8) kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_IMAGE_data[] = 
{
    0xed, 0x53, 0x31, 0x6c, 0xd3, 0x40, 0x14, 0xfd, 0x77, 0x17, 0xfb, 0x5c, 0x11, 0xb5, 0x89, 0x1c,
    0xe4, 0x42, 0x91, 0x0a, 0x32, 0x52, 0x3b, 0x22, 0x25, 0x8d, 0x0b, 0xea, 0x10, 0x31, 0xb4, 0x8c,
    0x80, 0x84, 0xca, 0x06, 0xc2, 0xa5, 0x20, 0x96, 0x2e, 0xa0, 0xae, 0xa9, 0x4c, 0x6b, 0x84, 0x58,
    0xc0, 0x52, 0x82, 0x14, 0x24, 0x3a, 0xb5, 0x4c, 0x08, 0xc9, 0xa5, 0xae, 0xc4, 0xc2, 0x00, 0xcd,
    0xc0, 0x82, 0x84, 0x20, 0x0c, 0x0c, 0x2d, 0xa6, 0x4d, 0x07, 0x86, 0x16, 0x68, 0x1a, 0x15, 0x68,
    0xf8, 0xb6, 0x2f, 0x65, 0x60, 0x66, 0xf3, 0x93, 0xac, 0xe7, 0xfb, 0xff, 0xdd, 0xff, 0xf7, 0xff,
    0xdd, 0xef, 0x21, 0x29, 0x50, 0xd3, 0x84, 0xaa, 0x84, 0x14, 0x26, 0xa0, 0x08, 0x13, 0x00, 0x50,
    0xe6, 0x00, 0x8e, 0x5d, 0x1d, 0xb2, 0x3c, 0x0e, 0x68, 0xdf, 0xd5, 0x87, 0xd2, 0x09, 0x6b, 0x9e,
    0x83, 0x3e, 0x38, 0x95, 0xd0, 0x73, 0xf7, 0x13, 0x6a, 0xd9, 0xa3, 0xe6, 0x69, 0x4e, 0x85, 0xee,
    0x54, 0xa0, 0x0b, 0xfc, 0xce, 0x8e, 0x06, 0x66, 0x89, 0x43, 0x19, 0x63, 0xd4, 0x8b, 0x2f, 0x7f,
    0x46, 0xfe, 0x91, 0x93, 0x6d, 0xbf, 0x5a, 0x9e, 0xa4, 0xfe, 0x79, 0xbe, 0x27, 0xfc, 0xbd, 0x62,
    0xbf, 0x11, 0xe6, 0x91, 0x4e, 0x24, 0xd5, 0x83, 0xb7, 0x98, 0x35, 0xc7, 0xdb, 0xf9, 0xf3, 0x81,
    0xbd, 0xf2, 0x61, 0x18, 0x9c, 0xd2, 0x32, 0x30, 0xdc, 0xaf, 0xc9, 0x00, 0xaa, 0xbc, 0x3c, 0xe0,
    0xda, 0x73, 0xc0, 0xf2, 0x32, 0x58, 0xcf, 0xed, 0xae, 0x1e, 0xce, 0xc1, 0x7f, 0xf8, 0xba, 0x89,
    0xff, 0xe8, 0x4b, 0x81, 0xff, 0xe0, 0x75, 0x53, 0xe8, 0xb2, 0x91, 0x26, 0xf4, 0x37, 0xd8, 0x12,
    0xef, 0xf0, 0xc7, 0x3e, 0x36, 0x50, 0x47, 0x7b, 0x46, 0x47, 0x40, 0xd5, 0xaa, 0xe0, 0xc9, 0x14,
    0xb2, 0xbc, 0x06, 0xae, 0xb4, 0x04, 0xfe, 0xb8, 0xf7, 0xc3, 0x5a, 0xb4, 0x15, 0xb6, 0xc8, 0xb9,
    0xbf, 0xb2, 0xfc, 0xc3, 0x5a, 0xb0, 0x49, 0x76, 0xa6, 0x04, 0xfe, 0x8d, 0xea, 0x37, 0x95, 0xbf,
    0xfd, 0xae, 0x6a, 0x93, 0x2c, 0x88, 0xcb, 0xf2, 0xc3, 0x39, 0x95, 0xef, 0xb4, 0x3c, 0xe5, 0x57,
    0x2b, 0xcb, 0xd7, 0xc1, 0xf9, 0xd8, 0x4d, 0xcc, 0x8d, 0x0d, 0x8c, 0x33, 0x57, 0xf0, 0xc7, 0xf8,
    0x96, 0xff, 0x79, 0x6d, 0x93, 0x79, 0x76, 0x52, 0x4d, 0x60, 0x3d, 0x99, 0x9b, 0x4c, 0xe5, 0x1a,
    0x64, 0x6f, 0xcf, 0x80, 0xbe, 0x50, 0x4f, 0x3a, 0xd2, 0x14, 0xd6, 0x99, 0x10, 0xfd, 0x29, 0x74,
    0xf8, 0x49, 0xe9, 0xab, 0x95, 0x9f, 0x02, 0xbf, 0x59, 0xfa, 0xea, 0xf0, 0x14, 0x18, 0x9c, 0x80,
    0x53, 0xe6, 0x54, 0x1f, 0x24, 0xa0, 0x0f, 0xad, 0x82, 0x7e, 0x82, 0x50, 0x3d, 0xb7, 0x85, 0xeb,
    0x55, 0xaa, 0x0f, 0x11, 0xea, 0xce, 0xcc, 0xe0, 0x7e, 0x42, 0xea, 0x45, 0x19, 0x98, 0x21, 0x51,
    0x47, 0xde, 0xc4, 0xbc, 0x6b, 0x60, 0x19, 0x9c, 0x9a, 0xbf, 0x1a, 0x94, 0x19, 0x36, 0xb8, 0x76,
    0x1d, 0x7b, 0xc7, 0x5b, 0x2e, 0xdf, 0x06, 0x6b, 0x91, 0x83, 0x61, 0xbf, 0x39, 0xa7, 0xf2, 0xda,
    0x96, 0x2a, 0x01, 0x98, 0x8f, 0xd6, 0xc1, 0xf2, 0xb0, 0x4f, 0x52, 0x0a, 0xf4, 0x67, 0xdb, 0x70,
    0xad, 0x88, 0x7d, 0x42, 0x7b, 0xc0, 0x56, 0x7e, 0x9a, 0x58, 0x17, 0xa7, 0xf1, 0xbc, 0xa8, 0x1b,
    0xd7, 0xc8, 0xbe, 0x7d, 0xf4, 0x2a, 0x63, 0x79, 0x89, 0xbe, 0xb0, 0xeb, 0x34, 0x38, 0x9b, 0x53,
    0xe3, 0xd4, 0xb5, 0xab, 0x10, 0xc4, 0xc7, 0x5e, 0x11, 0xf3, 0xcb, 0x32, 0x65, 0x4b, 0x36, 0xcd,
    0x2a, 0x32, 0x31, 0xf7, 0x64, 0xd2, 0x8e, 0x8f, 0x9a, 0x82, 0x6b, 0xd7, 0xf6, 0xf3, 0xf4, 0x60,
    0x1f, 0x26, 0x36, 0x1f, 0xef, 0x46, 0x77, 0x6b, 0x1f, 0x0e, 0xef, 0x9c, 0x43, 0x87, 0x3e, 0xbb,
    0x7f, 0xdf, 0x87, 0x02, 0x9b, 0x3e, 0x5b, 0x17, 0xeb, 0x91, 0x43, 0x91, 0xa6, 0x90, 0xfe, 0x6b,
    0xb3, 0xbb, 0x23, 0xdb, 0xd9, 0x50, 0x17, 0xc4, 0x75, 0x34, 0x39, 0x3a, 0x53, 0x7d, 0x1e, 0xdc,
    0xfa, 0x36, 0xbe, 0xc1, 0x9d, 0x2e, 0xf3, 0x6e, 0xf8, 0x6e, 0x5b, 0x7a, 0x2e, 0x2d, 0x7a, 0x0d,
    0xef, 0x34, 0x05, 0x6b, 0x55, 0x94, 0x83, 0x6c, 0x50, 0x01, 0x67, 0x74, 0x1a, 0x2a, 0x36, 0xe6,
    0x3a, 0xda, 0x14, 0x71, 0x79, 0x26, 0x8c, 0x7b, 0x61, 0x9a, 0x56, 0xf0, 0xad, 0xe9, 0xb3, 0xf3,
    0xa0, 0x4e, 0x6a, 0xe0, 0x4d, 0x02, 0xb8, 0x8d, 0x06, 0x78, 0x78, 0x2f, 0x26, 0x70, 0x19, 0x75,
    0x57, 0xdc, 0xf5, 0x06, 0x44, 0xb1, 0xce, 0xa4, 0x1d, 0xdc, 0xab, 0x31, 0xfc, 0x67, 0x2c, 0xad,
    0x75, 0x22, 0x77, 0x76, 0xa6, 0x71, 0x86, 0x28, 0xeb, 0x93, 0xc2, 0xfd, 0xfe, 0x11, 0xbe, 0xc7,
    0x0c, 0x86, 0x79, 0x34, 0xc2, 0x8c, 0x4e, 0xe4, 0x1d, 0xfc, 0xde, 0x07, 0xb9, 0x53, 0x2f, 0xee,
    0x78, 0x60, 0xde, 0xad, 0x02, 0x1b, 0x50, 0xb0, 0x2f, 0xbd, 0xbf, 0x31, 0xf6, 0xe5, 0x89, 0xcd,
    0x4b, 0xbb, 0x41, 0x4d, 0xa2, 0xfe, 0x03, 0x2c, 0x98, 0x9b, 0xbe, 0x60, 0xae, 0xfa, 0xc0, 0xbf,
    0x87, 0x73, 0x23, 0xec, 0xed, 0x79, 0xaa, 0xd4, 0xfa, 0xa2, 0x99, 0xfb, 0xb7, 0xde, 0x57, 0x41,
    0x9c, 0x00, 0x4a, 0x22, 0x1d, 0xd0, 0xd1, 0x5c, 0x86, 0x86, 0xdc, 0x2c, 0x46, 0xeb, 0xe3, 0x4f,
    0x7a, 0x43, 0xee, 0x17, 0x3c, 0x2b, 0xe0, 0x5e, 0x3e, 0x6c, 0x1c, 0x7b, 0x7a, 0xea, 0x53, 0x92,
    0x08, 0x9d, 0x1c, 0x71, 0x42, 0xb0, 0x22, 0x38, 0x25, 0xfc, 0xfd, 0x82, 0xbb, 0x05, 0x67, 0x04,
    0x1f, 0x11, 0xbc, 0x42, 0x23, 0x06, 0x81, 0x8c, 0xe0, 0x96, 0x40, 0xdb, 0x0e, 0x29, 0xf1, 0x21,
    0xae, 0x1f, 0x80, 0x18, 0x31, 0x62, 0xc4, 0x88, 0x11, 0x23, 0x46, 0x8c, 0x18, 0x31, 0x62, 0xfc,
    0x07, 0xfc, 0x01, 0x6a, 0x8a, 0x29, 0x3b, 0x00, 0x10, 0x00, 0x00,
};
#endif // defined(BINDATA_INCLUDE_DATA)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)
BINDATA_STORAGE_PVT kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_IMAGE_storage_pvt;
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)
{
    4096,                                                              // uncompressed data size (bytes)
    811,                                                               // compressed data size (bytes)
    kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_IMAGE_data,      // compressed data pointer
    BINDATA_FLAG_COMPRESSED | BINDATA_FLAG_EXTERNAL_VMWARE,            // u32 flags (see bin_data_pvt.h)
    0,                                                                 // Unused
},
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)


#if defined(BINDATA_INCLUDE_DATA)
//
// FUNCTION: kgspGetBinArchiveGspRmBoot_TU102("BINDATA_LABEL_UCODE_DESC")
// FILE NAME: kernel/inc/gsprm/bin/g_gsprm_tu10x_riscv_desc.bin
// FILE TYPE: BINARY
// VAR NAME: N/A
// COMPRESSION: YES
// COMPLEX_STRUCT: NO
// DATA SIZE (bytes): 76
// COMPRESSED SIZE (bytes): 21
// EXTERNAL: NO
//
static BINDATA_CONST NV_DECLARE_ALIGNED(NvU8, 8) kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_DESC_data[] = 
{
    0x63, 0x61, 0x80, 0x80, 0x0e, 0x16, 0x08, 0x16, 0x60, 0x20, 0x1f, 0x00, 0x00, 0xf5, 0x7a, 0xd2,
    0x3d, 0x4c, 0x00, 0x00, 0x00,
};
#endif // defined(BINDATA_INCLUDE_DATA)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)
BINDATA_STORAGE_PVT kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_DESC_storage_pvt;
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)
{
    76,                                                                // uncompressed data size (bytes)
    21,                                                                // compressed data size (bytes)
    kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_DESC_data,       // compressed data pointer
    BINDATA_FLAG_COMPRESSED | BINDATA_FLAG_EXTERNAL_VMWARE,            // u32 flags (see bin_data_pvt.h)
    0,                                                                 // Unused
},
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)


#if defined(BINDATA_INCLUDE_ARCHIVE)
//
// Bindata Archive structure
//
static const BINDATA_ARCHIVE __kgspGetBinArchiveGspRmBoot_TU102 =
{
    2,  // entryNum
    {
        // entries[] : { label, pBinStorage }
        { BINDATA_LABEL_UCODE_IMAGE     , (const PBINDATA_STORAGE) &g_bindata_pvt[__idx_kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_IMAGE] },
        { BINDATA_LABEL_UCODE_DESC      , (const PBINDATA_STORAGE) &g_bindata_pvt[__idx_kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_DESC] },
    }
};

#endif // defined(BINDATA_INCLUDE_ARCHIVE)



#if defined(BINDATA_INCLUDE_FUNCTION)
const BINDATA_ARCHIVE *kgspGetBinArchiveGspRmBoot_TU102(struct KernelGsp *pKernelGsp)
{
    return &__kgspGetBinArchiveGspRmBoot_TU102;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION)


#if defined(BINDATA_INCLUDE_FUNCTION_STUB)
const BINDATA_ARCHIVE *kgspGetBinArchiveGspRmBoot_TU102(struct KernelGsp *pKernelGsp)
{
    return NULL;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION_STUB)





