// Generated by the protocol buffer compiler.  DO NOT EDIT!

#ifndef G_GSP_PB_H__
#define G_GSP_PB_H__

#include "g_crashcat_pb.h"

extern const PRB_MSG_DESC prb_messages_gsp[];

// Message descriptor pointers
#define GSP_RPCENTRY (&prb_messages_gsp[0])
#define GSP_RPCHISTORYCPUTOGSP (&prb_messages_gsp[1])
#define GSP_RPCHISTORYGSPTOCPU (&prb_messages_gsp[2])
#define GSP_RPCDEBUGINFO (&prb_messages_gsp[3])
#define GSP_XIDREPORT (&prb_messages_gsp[4])
#define GSP_RPCPERF (&prb_messages_gsp[5])

// Message maximum lengths
// Does not include repeated fields, strings and byte arrays.
#define GSP_RPCENTRY_LEN 80
#define GSP_RPCHISTORYCPUTOGSP_LEN 84
#define GSP_RPCHISTORYGSPTOCPU_LEN 84
#define GSP_RPCDEBUGINFO_LEN 260
#define GSP_XIDREPORT_LEN 843
#define GSP_RPCPERF_LEN 92

extern const PRB_FIELD_DESC prb_fields_gsp_rpcentry[];

// 'RpcEntry' field descriptor pointers
#define GSP_RPCENTRY_HISTORYINDEX (&prb_fields_gsp_rpcentry[0])
#define GSP_RPCENTRY_FUNCTION (&prb_fields_gsp_rpcentry[1])
#define GSP_RPCENTRY_RPCNAME (&prb_fields_gsp_rpcentry[2])
#define GSP_RPCENTRY_DATA0 (&prb_fields_gsp_rpcentry[3])
#define GSP_RPCENTRY_DATA1 (&prb_fields_gsp_rpcentry[4])
#define GSP_RPCENTRY_STARTTIMESTAMP (&prb_fields_gsp_rpcentry[5])
#define GSP_RPCENTRY_ENDTIMESTAMP (&prb_fields_gsp_rpcentry[6])
#define GSP_RPCENTRY_DURATION (&prb_fields_gsp_rpcentry[7])
#define GSP_RPCENTRY_SEQUENCE (&prb_fields_gsp_rpcentry[8])

// 'RpcEntry' field lengths
#define GSP_RPCENTRY_HISTORYINDEX_LEN 5
#define GSP_RPCENTRY_FUNCTION_LEN 10
#define GSP_RPCENTRY_RPCNAME_LEN 1
#define GSP_RPCENTRY_DATA0_LEN 10
#define GSP_RPCENTRY_DATA1_LEN 10
#define GSP_RPCENTRY_STARTTIMESTAMP_LEN 10
#define GSP_RPCENTRY_ENDTIMESTAMP_LEN 10
#define GSP_RPCENTRY_DURATION_LEN 10
#define GSP_RPCENTRY_SEQUENCE_LEN 5

extern const PRB_FIELD_DESC prb_fields_gsp_rpchistorycputogsp[];

// 'RpcHistoryCpuToGsp' field descriptor pointers
#define GSP_RPCHISTORYCPUTOGSP_RPCENTRY (&prb_fields_gsp_rpchistorycputogsp[0])

// 'RpcHistoryCpuToGsp' field lengths
#define GSP_RPCHISTORYCPUTOGSP_RPCENTRY_LEN 83

extern const PRB_FIELD_DESC prb_fields_gsp_rpchistorygsptocpu[];

// 'RpcHistoryGspToCpu' field descriptor pointers
#define GSP_RPCHISTORYGSPTOCPU_RPCENTRY (&prb_fields_gsp_rpchistorygsptocpu[0])

// 'RpcHistoryGspToCpu' field lengths
#define GSP_RPCHISTORYGSPTOCPU_RPCENTRY_LEN 83

extern const PRB_FIELD_DESC prb_fields_gsp_rpcdebuginfo[];

// 'RpcDebugInfo' field descriptor pointers
#define GSP_RPCDEBUGINFO_ACTIVERPC (&prb_fields_gsp_rpcdebuginfo[0])
#define GSP_RPCDEBUGINFO_RPCHISTORYCPUTOGSP (&prb_fields_gsp_rpcdebuginfo[1])
#define GSP_RPCDEBUGINFO_RPCHISTORYGSPTOCPU (&prb_fields_gsp_rpcdebuginfo[2])

// 'RpcDebugInfo' field lengths
#define GSP_RPCDEBUGINFO_ACTIVERPC_LEN 83
#define GSP_RPCDEBUGINFO_RPCHISTORYCPUTOGSP_LEN 87
#define GSP_RPCDEBUGINFO_RPCHISTORYGSPTOCPU_LEN 87

extern const PRB_FIELD_DESC prb_fields_gsp_xidreport[];

// 'XidReport' field descriptor pointers
#define GSP_XIDREPORT_XID (&prb_fields_gsp_xidreport[0])
#define GSP_XIDREPORT_GPUINSTANCE (&prb_fields_gsp_xidreport[1])
#define GSP_XIDREPORT_BUILDID (&prb_fields_gsp_xidreport[2])
#define GSP_XIDREPORT_CRASHCATREPORT (&prb_fields_gsp_xidreport[3])
#define GSP_XIDREPORT_RPCDEBUGINFO (&prb_fields_gsp_xidreport[4])

// 'XidReport' field lengths
#define GSP_XIDREPORT_XID_LEN 5
#define GSP_XIDREPORT_GPUINSTANCE_LEN 5
#define GSP_XIDREPORT_BUILDID_LEN 1
#define GSP_XIDREPORT_CRASHCATREPORT_LEN 564
#define GSP_XIDREPORT_RPCDEBUGINFO_LEN 263

extern const PRB_FIELD_DESC prb_fields_gsp_rpcperf[];

// 'RpcPerf' field descriptor pointers
#define GSP_RPCPERF_MSG (&prb_fields_gsp_rpcperf[0])
#define GSP_RPCPERF_GPUINSTANCE (&prb_fields_gsp_rpcperf[1])
#define GSP_RPCPERF_ACTIVERPC (&prb_fields_gsp_rpcperf[2])

// 'RpcPerf' field lengths
#define GSP_RPCPERF_MSG_LEN 1
#define GSP_RPCPERF_GPUINSTANCE_LEN 5
#define GSP_RPCPERF_ACTIVERPC_LEN 83

extern const PRB_SERVICE_DESC prb_services_gsp[];

// Service descriptor pointers

#endif // G_GSP_PB_H__
