/*
 * SPDX-FileCopyrightText: Copyright (c) 2016-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


/* THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT! */



#if defined(BINDATA_INCLUDE_DATA)
//
// FUNCTION: spdmGetBinArchiveIndividualL3Certificate_GR102("BINDATA_LABEL_CERTIFICATE_PEM")
// FILE NAME: inc/kernel/spdm/bin/rubin/gr102_l3_cert.pem.bin
// FILE TYPE: BINARY
// VAR NAME: N/A
// COMPRESSION: YES
// COMPLEX_STRUCT: NO
// DATA SIZE (bytes): 989
// COMPRESSED SIZE (bytes): 746
// EXTERNAL: NO
//
static BINDATA_CONST NV_DECLARE_ALIGNED(NvU8, 8) spdmBinArchiveIndividualL3Certificate_GR102_BINDATA_LABEL_CERTIFICATE_PEM_data[] = 
{
    0x65, 0x53, 0xc9, 0x72, 0xab, 0x48, 0x10, 0xbc, 0xf3, 0x15, 0x73, 0x77, 0x38, 0x2c, 0x23, 0x90,
    0xe1, 0xf0, 0x0e, 0x55, 0xd0, 0x42, 0x20, 0x1a, 0xd4, 0x6c, 0x92, 0xb8, 0xb1, 0x18, 0x24, 0x16,
    0xed, 0xd0, 0xc0, 0xd7, 0x8f, 0x64, 0xc7, 0xcc, 0x1c, 0xa6, 0x23, 0xf2, 0x52, 0x51, 0x99, 0x5d,
    0x4b, 0xd6, 0xfb, 0xfb, 0xf3, 0x21, 0x31, 0x4c, 0xe7, 0x2f, 0x8d, 0x78, 0x81, 0xb9, 0x34, 0x35,
    0x08, 0xc8, 0x2b, 0xf8, 0x2e, 0x50, 0xd3, 0xd4, 0x6e, 0x81, 0xa6, 0x41, 0xe5, 0x96, 0xc0, 0x4d,
    0x84, 0xd2, 0x64, 0x99, 0x93, 0x77, 0x07, 0x7f, 0xed, 0x7d, 0x9a, 0xc7, 0x8f, 0x86, 0x41, 0xaf,
    0x5a, 0xdc, 0xdc, 0x54, 0xb0, 0xc6, 0xb2, 0xbc, 0x1e, 0xea, 0xca, 0xdd, 0x30, 0xa6, 0xc3, 0x04,
    0x2a, 0xf5, 0x24, 0x2e, 0xac, 0x60, 0xaf, 0x47, 0xcf, 0x80, 0x8e, 0x91, 0x1b, 0xd5, 0x0d, 0xf1,
    0x43, 0x52, 0x3e, 0x99, 0xc3, 0x76, 0x59, 0x96, 0xfe, 0xd6, 0x6b, 0xd2, 0x93, 0x77, 0xc9, 0x57,
    0xf5, 0x60, 0x4c, 0x10, 0x63, 0xe9, 0x44, 0x08, 0x67, 0x4a, 0x6a, 0x79, 0xeb, 0x87, 0x9e, 0xc5,
    0x7c, 0xd4, 0x85, 0x74, 0x6e, 0xf1, 0x27, 0x0e, 0xb9, 0xd1, 0xf4, 0x69, 0x05, 0xe5, 0x92, 0xcf,
    0x46, 0x3a, 0x81, 0x48, 0x2b, 0xe0, 0x54, 0x7f, 0x01, 0x13, 0x03, 0x14, 0xd9, 0x0d, 0x6a, 0x99,
    0x06, 0xe6, 0xf4, 0x82, 0x13, 0xd4, 0x9f, 0x6e, 0x74, 0xe6, 0xd1, 0x44, 0x6e, 0x02, 0xd5, 0x6a,
    0x03, 0x3e, 0x43, 0x02, 0x9c, 0x1f, 0x83, 0x26, 0xb6, 0x3c, 0xd2, 0xa0, 0x49, 0x72, 0x9f, 0x06,
    0x30, 0x9a, 0x4b, 0x1c, 0xd3, 0x79, 0x7c, 0xc9, 0xc4, 0xa7, 0x78, 0x1b, 0x8d, 0x26, 0x69, 0x74,
    0xe6, 0xff, 0x23, 0x4c, 0x52, 0x8a, 0xb5, 0x21, 0xbc, 0xc8, 0x5a, 0xc9, 0xfd, 0xff, 0xc8, 0x4e,
    0x9f, 0x9d, 0xb0, 0xcf, 0xda, 0xe5, 0x2c, 0xd9, 0xaa, 0x1d, 0x65, 0x77, 0xae, 0xb1, 0x9f, 0x36,
    0x0d, 0xc2, 0xad, 0x28, 0x9c, 0x50, 0xa4, 0x08, 0x06, 0x8e, 0x57, 0xc3, 0xa7, 0x92, 0x2a, 0x40,
    0x49, 0x0c, 0xf4, 0x3b, 0x44, 0x80, 0x23, 0x88, 0x26, 0xe0, 0x6a, 0x60, 0x8b, 0x4b, 0xeb, 0x5f,
    0xe4, 0xf6, 0xbc, 0x8e, 0xc5, 0xca, 0x79, 0xcb, 0xec, 0x31, 0x92, 0x62, 0xed, 0x6b, 0xba, 0xe7,
    0x07, 0xf7, 0x4b, 0xa2, 0x51, 0x80, 0x3b, 0xc9, 0xde, 0x3d, 0xea, 0x7d, 0x7f, 0x8c, 0x6c, 0xa1,
    0x88, 0x8f, 0x83, 0x6d, 0x51, 0x54, 0x4f, 0xdb, 0x2a, 0xb7, 0x3c, 0x95, 0x60, 0x9f, 0x16, 0xdb,
    0xdb, 0x8e, 0x54, 0x5f, 0x43, 0x71, 0xbd, 0xae, 0xd5, 0xd0, 0xcb, 0x25, 0x5a, 0xe0, 0x79, 0x76,
    0x13, 0xbd, 0xfa, 0x53, 0x6a, 0x76, 0xdb, 0xc1, 0x4f, 0x12, 0x57, 0xaa, 0xc9, 0xd7, 0x5b, 0xbf,
    0x5b, 0x08, 0x76, 0x96, 0xbd, 0x29, 0xf6, 0x46, 0x91, 0x8b, 0x38, 0x09, 0xe5, 0x85, 0x5e, 0x5c,
    0x5d, 0x14, 0x27, 0x0d, 0x45, 0x1d, 0x36, 0xaf, 0xa9, 0xaf, 0xbc, 0x67, 0xc1, 0x85, 0x42, 0x30,
    0x00, 0x1d, 0xd8, 0xea, 0x83, 0x82, 0xf4, 0x9a, 0x59, 0xae, 0x73, 0x82, 0x1f, 0x9c, 0x11, 0xe1,
    0xb5, 0x7b, 0xac, 0x40, 0xf9, 0x4d, 0x56, 0x88, 0x13, 0xc0, 0x44, 0x75, 0xa3, 0xb4, 0x17, 0xf0,
    0x38, 0x1c, 0x1f, 0xe7, 0x7c, 0xe5, 0x71, 0xf7, 0xa8, 0xf4, 0xfb, 0xb9, 0x75, 0xb7, 0x5b, 0xb9,
    0x4e, 0x76, 0xb4, 0x4b, 0x4f, 0xf1, 0x25, 0x36, 0x9a, 0x83, 0xdd, 0x3a, 0xbd, 0x90, 0xfa, 0x6a,
    0x95, 0xb5, 0xbc, 0x4f, 0x75, 0xf3, 0x11, 0xcf, 0xcd, 0xe1, 0x7b, 0x55, 0x76, 0xaf, 0x64, 0xaa,
    0x67, 0x86, 0xa6, 0xdd, 0x0d, 0x60, 0xe1, 0x12, 0x9f, 0xbf, 0xa1, 0x76, 0xe7, 0xeb, 0x00, 0x4e,
    0x4f, 0x33, 0xdd, 0xb0, 0x24, 0x4b, 0x64, 0x19, 0x87, 0xfd, 0x3e, 0x15, 0x92, 0x95, 0x37, 0xcb,
    0xf4, 0x73, 0x6f, 0x8b, 0x4f, 0xa1, 0x39, 0x74, 0x69, 0xeb, 0x5d, 0xb2, 0x51, 0xee, 0xf2, 0xb6,
    0xa9, 0x93, 0x2d, 0xe9, 0xf6, 0xa2, 0xfa, 0xa0, 0x38, 0xfb, 0x2d, 0xbb, 0x64, 0x5b, 0x44, 0xaf,
    0x65, 0xca, 0xe6, 0x53, 0x62, 0xd6, 0x38, 0x1e, 0xaf, 0xa2, 0xa0, 0x0e, 0xd7, 0x64, 0x3e, 0xcd,
    0xf5, 0xfd, 0x7e, 0x54, 0xac, 0x09, 0x8a, 0x9f, 0x56, 0x7c, 0x4a, 0x0c, 0x1d, 0xb6, 0x25, 0xfa,
    0x91, 0x23, 0x15, 0x8a, 0x12, 0x68, 0xb9, 0x7c, 0x08, 0x25, 0xea, 0x2e, 0x6e, 0xd1, 0xb2, 0xb6,
    0xce, 0x1e, 0x54, 0x33, 0xfd, 0xd7, 0xda, 0xc2, 0xbf, 0xde, 0xe6, 0xce, 0x19, 0x74, 0x6c, 0xa0,
    0x82, 0x1b, 0xb9, 0xda, 0x4a, 0xa5, 0xe1, 0xd9, 0x65, 0xa1, 0x6d, 0x9f, 0xf7, 0x94, 0xc8, 0x62,
    0x14, 0xb9, 0xa3, 0xd9, 0x35, 0xa6, 0x84, 0xd1, 0x86, 0x15, 0xed, 0x9b, 0x62, 0xcd, 0xf9, 0xe0,
    0xa0, 0xda, 0x09, 0xc5, 0x66, 0x7f, 0x52, 0x8d, 0x21, 0x5d, 0x1c, 0x4c, 0x35, 0x67, 0xf1, 0x43,
    0xe4, 0x4c, 0xa3, 0x4c, 0x3b, 0x16, 0x2c, 0xd6, 0x4a, 0xe5, 0xfb, 0x31, 0x85, 0xb7, 0xdb, 0x75,
    0x37, 0xcf, 0xe4, 0x42, 0x79, 0x9b, 0xa6, 0xbc, 0x39, 0xf0, 0x32, 0xe8, 0xf5, 0xea, 0x7b, 0x09,
    0x48, 0x86, 0xe4, 0x30, 0x08, 0x3d, 0x8b, 0xef, 0xe1, 0x1d, 0x8f, 0x32, 0xa8, 0xfe, 0x8d, 0x2f,
    0xdb, 0x64, 0xd1, 0x7d, 0x7d, 0x9b, 0xe6, 0x1f, 0xe1, 0xe7, 0x58, 0x89, 0xa3, 0xff, 0xff, 0x80,
    0xff, 0x06, 0x07, 0xda, 0x52, 0xd5, 0xdd, 0x03, 0x00, 0x00,
};
#endif // defined(BINDATA_INCLUDE_DATA)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)
BINDATA_STORAGE_PVT spdmBinArchiveIndividualL3Certificate_GR102_BINDATA_LABEL_CERTIFICATE_PEM_storage_pvt;
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)
{
    989,                                                               // uncompressed data size (bytes)
    746,                                                               // compressed data size (bytes)
    spdmBinArchiveIndividualL3Certificate_GR102_BINDATA_LABEL_CERTIFICATE_PEM_data, // compressed data pointer
    BINDATA_FLAG_COMPRESSED | BINDATA_FLAG_EXTERNAL_VMWARE,            // u32 flags (see bin_data_pvt.h)
    0,                                                                 // Unused
},
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)


#if defined(BINDATA_INCLUDE_ARCHIVE)
//
// Bindata Archive structure
//
static const BINDATA_ARCHIVE __spdmGetBinArchiveIndividualL3Certificate_GR102 =
{
    1,  // entryNum
    {
        // entries[] : { label, pBinStorage }
        { BINDATA_LABEL_CERTIFICATE_PEM , (const PBINDATA_STORAGE) &g_bindata_pvt[__idx_spdmBinArchiveIndividualL3Certificate_GR102_BINDATA_LABEL_CERTIFICATE_PEM] },
    }
};

#endif // defined(BINDATA_INCLUDE_ARCHIVE)



#if defined(BINDATA_INCLUDE_FUNCTION)
const BINDATA_ARCHIVE *spdmGetBinArchiveIndividualL3Certificate_GR102(struct Spdm *pSpdm)
{
    return &__spdmGetBinArchiveIndividualL3Certificate_GR102;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION)


#if defined(BINDATA_INCLUDE_FUNCTION_STUB)
const BINDATA_ARCHIVE *spdmGetBinArchiveIndividualL3Certificate_GR102(struct Spdm *pSpdm)
{
    return NULL;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION_STUB)





