/*
 * SPDX-FileCopyrightText: Copyright (c) 2016-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


/* THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT! */



#if defined(BINDATA_INCLUDE_DATA)
//
// FUNCTION: ksec2GetBinArchiveBlUcode_TU102("BINDATA_LABEL_UCODE_IMAGE")
// FILE NAME: kernel/inc/sec2/bin/g_sec2_bl_gp10x.h
// FILE TYPE: TEXT
// VAR NAME: sec2_bl_gp10x
// COMPRESSION: YES
// COMPLEX_STRUCT: NO
// DATA SIZE (bytes): 768
// COMPRESSED SIZE (bytes): 303
// EXTERNAL: NO
//
static BINDATA_CONST NV_DECLARE_ALIGNED(NvU8, 8) ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_IMAGE_data[] = 
{
    0xed, 0x8f, 0xbf, 0x4e, 0xc2, 0x50, 0x14, 0xc6, 0x3f, 0x4a, 0x8b, 0xb7, 0x52, 0xa1, 0xad, 0x0e,
    0x24, 0x4c, 0x3e, 0x42, 0x77, 0xd3, 0xb8, 0xfa, 0x06, 0xba, 0xf9, 0x0e, 0xee, 0xee, 0x2c, 0x5a,
    0x93, 0x1a, 0x6f, 0x41, 0x5a, 0x07, 0xe2, 0x03, 0x40, 0x52, 0x19, 0x7a, 0x47, 0x66, 0x1d, 0x70,
    0xf4, 0xcf, 0xe2, 0xc0, 0x04, 0x26, 0x57, 0xd3, 0x96, 0xf4, 0xea, 0xad, 0x38, 0x1a, 0x67, 0x07,
    0xcf, 0x70, 0xbe, 0xe4, 0xcb, 0x39, 0xdf, 0xf9, 0x9d, 0x3b, 0xec, 0x02, 0x42, 0x05, 0x1b, 0x1c,
    0x9b, 0x05, 0x52, 0xa5, 0x03, 0x80, 0x86, 0x3a, 0xed, 0xaf, 0x67, 0x36, 0xed, 0xd9, 0xf1, 0x0b,
    0xf1, 0x9f, 0x5a, 0xf4, 0xb2, 0x59, 0x08, 0x8d, 0x76, 0x37, 0xe9, 0xd9, 0x16, 0xf5, 0x1a, 0xe2,
    0x1d, 0xf1, 0x3d, 0x11, 0x8f, 0xf8, 0x1a, 0x8e, 0x88, 0x9f, 0x11, 0x5f, 0x18, 0x45, 0xa8, 0x15,
    0x91, 0x26, 0x7a, 0xdf, 0xae, 0x41, 0x83, 0x3a, 0x9b, 0x26, 0xd1, 0xc2, 0x3d, 0x2a, 0x90, 0xf7,
    0xd5, 0x71, 0x80, 0x0a, 0xc6, 0x0f, 0xb2, 0x5d, 0x47, 0x9c, 0xbc, 0xa6, 0x6b, 0x23, 0xe0, 0x90,
    0xef, 0x28, 0xa7, 0x40, 0xad, 0x63, 0x02, 0x72, 0x97, 0xf1, 0x3c, 0xab, 0xa5, 0x55, 0xb9, 0x5f,
    0x11, 0xdd, 0x55, 0x4c, 0xb8, 0x31, 0x9c, 0x25, 0xcf, 0x0b, 0xf7, 0xe6, 0xe7, 0x8c, 0xd5, 0xa9,
    0x06, 0x0d, 0xf4, 0x98, 0x9b, 0xf1, 0x39, 0x49, 0x42, 0x4f, 0xf6, 0x12, 0xa3, 0xfb, 0x2b, 0x9c,
    0x49, 0x43, 0x2b, 0x7e, 0x23, 0xfe, 0xac, 0x55, 0xba, 0x17, 0x90, 0x6f, 0x8a, 0x39, 0x18, 0x77,
    0xb7, 0x05, 0x4e, 0x24, 0x50, 0x22, 0xbc, 0x3c, 0xd3, 0xc6, 0x1f, 0xe5, 0xa5, 0x25, 0x27, 0xf3,
    0xb4, 0xaa, 0x93, 0xdb, 0x00, 0x0b, 0x9f, 0xf0, 0x7a, 0x5b, 0x12, 0x57, 0xf6, 0x3c, 0xe4, 0x11,
    0x9a, 0x8a, 0xbe, 0x5f, 0x0a, 0xd1, 0x55, 0x29, 0xa3, 0x2b, 0x18, 0xc3, 0x76, 0x36, 0x71, 0x0f,
    0x04, 0xb2, 0x89, 0x7c, 0xcf, 0xe6, 0x8e, 0xc9, 0x1d, 0x8b, 0x3b, 0x36, 0xf3, 0x25, 0x10, 0x1b,
    0x2c, 0x2d, 0xfc, 0xd7, 0xdf, 0xaa, 0x4f, 0x55, 0x99, 0xb8, 0xc4, 0x00, 0x03, 0x00, 0x00,
};
#endif // defined(BINDATA_INCLUDE_DATA)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)
BINDATA_STORAGE_PVT ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_IMAGE_storage_pvt;
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)
{
    768,                                                               // uncompressed data size (bytes)
    303,                                                               // compressed data size (bytes)
    ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_IMAGE_data,       // compressed data pointer
    BINDATA_FLAG_COMPRESSED | BINDATA_FLAG_EXTERNAL_VMWARE,            // u32 flags (see bin_data_pvt.h)
    0,                                                                 // Unused
},
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)


#if defined(BINDATA_INCLUDE_DATA)
//
// FUNCTION: ksec2GetBinArchiveBlUcode_TU102("BINDATA_LABEL_UCODE_DESC")
// FILE NAME: kernel/inc/sec2/bin/g_sec2_bl_gp10x.h
// FILE TYPE: TEXT
// VAR NAME: sec2_bl_gp10x_desc
// COMPRESSION: NO
// COMPLEX_STRUCT: YES
// DATA SIZE (bytes): sizeof(ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC_data)
// COMPRESSED SIZE (bytes): N/A
// EXTERNAL: NO
//
static const RM_FLCN_BL_DESC ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC_data = {
    0xfd, 
    0, 
    {
        0x0, 
        0x200, 
        0x200, 
        0x100  
    }
};
#endif // defined(BINDATA_INCLUDE_DATA)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)
BINDATA_STORAGE_PVT ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC_storage_pvt;
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DECL)

#if defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)
{
    sizeof(ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC_data), // uncompressed data size (bytes)
    sizeof(ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC_data), // compressed data size (bytes)
    &ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC_data,       // compressed data pointer
    0,                                                                 // u32 flags (see bin_data_pvt.h)
    0,                                                                 // Unused
},
#endif // defined(BINDATA_INCLUDE_STORAGE_PVT_DEFN)


#if defined(BINDATA_INCLUDE_ARCHIVE)
//
// Bindata Archive structure
//
static const BINDATA_ARCHIVE __ksec2GetBinArchiveBlUcode_TU102 =
{
    2,  // entryNum
    {
        // entries[] : { label, pBinStorage }
        { BINDATA_LABEL_UCODE_IMAGE     , (const PBINDATA_STORAGE) &g_bindata_pvt[__idx_ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_IMAGE] },
        { BINDATA_LABEL_UCODE_DESC      , (const PBINDATA_STORAGE) &g_bindata_pvt[__idx_ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC] },
    }
};

#endif // defined(BINDATA_INCLUDE_ARCHIVE)



#if defined(BINDATA_INCLUDE_FUNCTION)
const BINDATA_ARCHIVE *ksec2GetBinArchiveBlUcode_TU102(struct KernelSec2 *pKernelSec2)
{
    return &__ksec2GetBinArchiveBlUcode_TU102;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION)


#if defined(BINDATA_INCLUDE_FUNCTION_STUB)
const BINDATA_ARCHIVE *ksec2GetBinArchiveBlUcode_TU102(struct KernelSec2 *pKernelSec2)
{
    return NULL;
}
#endif // defined(BINDATA_INCLUDE_FUNCTION_STUB)





