// Generated by the protocol buffer compiler.  DO NOT EDIT!

#ifndef G_CRASHCAT_PB_H__
#define G_CRASHCAT_PB_H__


extern const PRB_ENUM_DESC prb_enums_crashcat_report_riscvmode;

// 'Report.RiscvMode' enumeration values
#define CRASHCAT_REPORT_MODE_X 1
#define CRASHCAT_REPORT_MODE_M 2
#define CRASHCAT_REPORT_MODE_S 3
#define CRASHCAT_REPORT_MODE_U 4

extern const PRB_ENUM_DESC prb_enums_crashcat_report_causetype;

// 'Report.CauseType' enumeration values
#define CRASHCAT_REPORT_UNKNOWN 1
#define CRASHCAT_REPORT_EXCEPTION 2
#define CRASHCAT_REPORT_TIMEOUT 3
#define CRASHCAT_REPORT_PANIC 4

extern const PRB_ENUM_DESC prb_enums_crashcat_report_panicreasontype;

// 'Report.PanicReasonType' enumeration values
#define CRASHCAT_REPORT_LIBOSPANICREASONUNSPECIFIED 1
#define CRASHCAT_REPORT_LIBOSPANICREASONUNHANDLEDSTATE 2
#define CRASHCAT_REPORT_LIBOSPANICREASONINVALIDCONFIGURATION 3
#define CRASHCAT_REPORT_LIBOSPANICREASONFATALHARDWAREERROR 4
#define CRASHCAT_REPORT_LIBOSPANICREASONINSUFFICIENTRESOURCES 5
#define CRASHCAT_REPORT_LIBOSPANICREASONTIMEOUT 6
#define CRASHCAT_REPORT_LIBOSPANICREASONENVCALLFAILED 7
#define CRASHCAT_REPORT_LIBOSPANICREASONASANMEMORYERROR 8
#define CRASHCAT_REPORT_LIBOSPANICREASONPROGRAMMINGERROR 9
#define CRASHCAT_REPORT_LIBOSPANICREASONASSERTIONFAILED 10
#define CRASHCAT_REPORT_LIBOSPANICREASONTRAPKERNELPANIC 11
#define CRASHCAT_REPORT_LIBOSPANICREASONTRAPINSTRUCTION 12

extern const PRB_MSG_DESC prb_messages_crashcat[];

// Message descriptor pointers
#define CRASHCAT_RISCV64CSRSTATE (&prb_messages_crashcat[0])
#define CRASHCAT_RISCV64GPRSTATE (&prb_messages_crashcat[1])
#define CRASHCAT_LOCALIOREGISTER (&prb_messages_crashcat[2])
#define CRASHCAT_REPORT (&prb_messages_crashcat[3])

// Message maximum lengths
// Does not include repeated fields, strings and byte arrays.
#define CRASHCAT_RISCV64CSRSTATE_LEN 88
#define CRASHCAT_RISCV64GPRSTATE_LEN 352
#define CRASHCAT_LOCALIOREGISTER_LEN 12
#define CRASHCAT_REPORT_LEN 561

extern const PRB_FIELD_DESC prb_fields_crashcat_riscv64csrstate[];

// 'Riscv64CsrState' field descriptor pointers
#define CRASHCAT_RISCV64CSRSTATE_HEADER (&prb_fields_crashcat_riscv64csrstate[0])
#define CRASHCAT_RISCV64CSRSTATE_XSTATUS (&prb_fields_crashcat_riscv64csrstate[1])
#define CRASHCAT_RISCV64CSRSTATE_XIE (&prb_fields_crashcat_riscv64csrstate[2])
#define CRASHCAT_RISCV64CSRSTATE_XIP (&prb_fields_crashcat_riscv64csrstate[3])
#define CRASHCAT_RISCV64CSRSTATE_XEPC (&prb_fields_crashcat_riscv64csrstate[4])
#define CRASHCAT_RISCV64CSRSTATE_XTVAL (&prb_fields_crashcat_riscv64csrstate[5])
#define CRASHCAT_RISCV64CSRSTATE_XCAUSE (&prb_fields_crashcat_riscv64csrstate[6])
#define CRASHCAT_RISCV64CSRSTATE_XSCRATCH (&prb_fields_crashcat_riscv64csrstate[7])

// 'Riscv64CsrState' field lengths
#define CRASHCAT_RISCV64CSRSTATE_HEADER_LEN 10
#define CRASHCAT_RISCV64CSRSTATE_XSTATUS_LEN 10
#define CRASHCAT_RISCV64CSRSTATE_XIE_LEN 10
#define CRASHCAT_RISCV64CSRSTATE_XIP_LEN 10
#define CRASHCAT_RISCV64CSRSTATE_XEPC_LEN 10
#define CRASHCAT_RISCV64CSRSTATE_XTVAL_LEN 10
#define CRASHCAT_RISCV64CSRSTATE_XCAUSE_LEN 10
#define CRASHCAT_RISCV64CSRSTATE_XSCRATCH_LEN 10

extern const PRB_FIELD_DESC prb_fields_crashcat_riscv64gprstate[];

// 'Riscv64GprState' field descriptor pointers
#define CRASHCAT_RISCV64GPRSTATE_HEADER (&prb_fields_crashcat_riscv64gprstate[0])
#define CRASHCAT_RISCV64GPRSTATE_RA (&prb_fields_crashcat_riscv64gprstate[1])
#define CRASHCAT_RISCV64GPRSTATE_SP (&prb_fields_crashcat_riscv64gprstate[2])
#define CRASHCAT_RISCV64GPRSTATE_GP (&prb_fields_crashcat_riscv64gprstate[3])
#define CRASHCAT_RISCV64GPRSTATE_TP (&prb_fields_crashcat_riscv64gprstate[4])
#define CRASHCAT_RISCV64GPRSTATE_T0 (&prb_fields_crashcat_riscv64gprstate[5])
#define CRASHCAT_RISCV64GPRSTATE_T1 (&prb_fields_crashcat_riscv64gprstate[6])
#define CRASHCAT_RISCV64GPRSTATE_T2 (&prb_fields_crashcat_riscv64gprstate[7])
#define CRASHCAT_RISCV64GPRSTATE_S0 (&prb_fields_crashcat_riscv64gprstate[8])
#define CRASHCAT_RISCV64GPRSTATE_S1 (&prb_fields_crashcat_riscv64gprstate[9])
#define CRASHCAT_RISCV64GPRSTATE_A0 (&prb_fields_crashcat_riscv64gprstate[10])
#define CRASHCAT_RISCV64GPRSTATE_A1 (&prb_fields_crashcat_riscv64gprstate[11])
#define CRASHCAT_RISCV64GPRSTATE_A2 (&prb_fields_crashcat_riscv64gprstate[12])
#define CRASHCAT_RISCV64GPRSTATE_A3 (&prb_fields_crashcat_riscv64gprstate[13])
#define CRASHCAT_RISCV64GPRSTATE_A4 (&prb_fields_crashcat_riscv64gprstate[14])
#define CRASHCAT_RISCV64GPRSTATE_A5 (&prb_fields_crashcat_riscv64gprstate[15])
#define CRASHCAT_RISCV64GPRSTATE_A6 (&prb_fields_crashcat_riscv64gprstate[16])
#define CRASHCAT_RISCV64GPRSTATE_A7 (&prb_fields_crashcat_riscv64gprstate[17])
#define CRASHCAT_RISCV64GPRSTATE_S2 (&prb_fields_crashcat_riscv64gprstate[18])
#define CRASHCAT_RISCV64GPRSTATE_S3 (&prb_fields_crashcat_riscv64gprstate[19])
#define CRASHCAT_RISCV64GPRSTATE_S4 (&prb_fields_crashcat_riscv64gprstate[20])
#define CRASHCAT_RISCV64GPRSTATE_S5 (&prb_fields_crashcat_riscv64gprstate[21])
#define CRASHCAT_RISCV64GPRSTATE_S6 (&prb_fields_crashcat_riscv64gprstate[22])
#define CRASHCAT_RISCV64GPRSTATE_S7 (&prb_fields_crashcat_riscv64gprstate[23])
#define CRASHCAT_RISCV64GPRSTATE_S8 (&prb_fields_crashcat_riscv64gprstate[24])
#define CRASHCAT_RISCV64GPRSTATE_S9 (&prb_fields_crashcat_riscv64gprstate[25])
#define CRASHCAT_RISCV64GPRSTATE_S10 (&prb_fields_crashcat_riscv64gprstate[26])
#define CRASHCAT_RISCV64GPRSTATE_S11 (&prb_fields_crashcat_riscv64gprstate[27])
#define CRASHCAT_RISCV64GPRSTATE_T3 (&prb_fields_crashcat_riscv64gprstate[28])
#define CRASHCAT_RISCV64GPRSTATE_T4 (&prb_fields_crashcat_riscv64gprstate[29])
#define CRASHCAT_RISCV64GPRSTATE_T5 (&prb_fields_crashcat_riscv64gprstate[30])
#define CRASHCAT_RISCV64GPRSTATE_T6 (&prb_fields_crashcat_riscv64gprstate[31])

// 'Riscv64GprState' field lengths
#define CRASHCAT_RISCV64GPRSTATE_HEADER_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_RA_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_SP_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_GP_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_TP_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_T0_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_T1_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_T2_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S0_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S1_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A0_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A1_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A2_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A3_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A4_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A5_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A6_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_A7_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S2_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S3_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S4_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S5_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S6_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S7_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S8_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S9_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S10_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_S11_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_T3_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_T4_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_T5_LEN 10
#define CRASHCAT_RISCV64GPRSTATE_T6_LEN 10

extern const PRB_FIELD_DESC prb_fields_crashcat_localioregister[];

// 'LocalIORegister' field descriptor pointers
#define CRASHCAT_LOCALIOREGISTER_ADDRESS (&prb_fields_crashcat_localioregister[0])
#define CRASHCAT_LOCALIOREGISTER_VALUE (&prb_fields_crashcat_localioregister[1])

// 'LocalIORegister' field lengths
#define CRASHCAT_LOCALIOREGISTER_ADDRESS_LEN 5
#define CRASHCAT_LOCALIOREGISTER_VALUE_LEN 5

extern const PRB_FIELD_DESC prb_fields_crashcat_report[];

// 'Report' field descriptor pointers
#define CRASHCAT_REPORT_MODE (&prb_fields_crashcat_report[0])
#define CRASHCAT_REPORT_CAUSE (&prb_fields_crashcat_report[1])
#define CRASHCAT_REPORT_LIBOSPANICREASON (&prb_fields_crashcat_report[2])
#define CRASHCAT_REPORT_PC (&prb_fields_crashcat_report[3])
#define CRASHCAT_REPORT_SOURCEPARTITION (&prb_fields_crashcat_report[4])
#define CRASHCAT_REPORT_SOURCEUCODEID (&prb_fields_crashcat_report[5])
#define CRASHCAT_REPORT_REPORTERPARTITION (&prb_fields_crashcat_report[6])
#define CRASHCAT_REPORT_REPORTERUCODEID (&prb_fields_crashcat_report[7])
#define CRASHCAT_REPORT_LIBOSVERSIONMAJOR (&prb_fields_crashcat_report[8])
#define CRASHCAT_REPORT_LIBOSVERSIONMINOR (&prb_fields_crashcat_report[9])
#define CRASHCAT_REPORT_LIBOSVERSIONCL (&prb_fields_crashcat_report[10])
#define CRASHCAT_REPORT_TIMESTAMP (&prb_fields_crashcat_report[11])
#define CRASHCAT_REPORT_CSRSTATE (&prb_fields_crashcat_report[12])
#define CRASHCAT_REPORT_GPRSTATE (&prb_fields_crashcat_report[13])
#define CRASHCAT_REPORT_STACKTRACE (&prb_fields_crashcat_report[14])
#define CRASHCAT_REPORT_PCTRACE (&prb_fields_crashcat_report[15])
#define CRASHCAT_REPORT_LOCALIOREGISTER (&prb_fields_crashcat_report[16])
#define CRASHCAT_REPORT_BUILDID (&prb_fields_crashcat_report[17])

// 'Report' field lengths
#define CRASHCAT_REPORT_MODE_LEN 2
#define CRASHCAT_REPORT_CAUSE_LEN 2
#define CRASHCAT_REPORT_LIBOSPANICREASON_LEN 2
#define CRASHCAT_REPORT_PC_LEN 10
#define CRASHCAT_REPORT_SOURCEPARTITION_LEN 5
#define CRASHCAT_REPORT_SOURCEUCODEID_LEN 5
#define CRASHCAT_REPORT_REPORTERPARTITION_LEN 5
#define CRASHCAT_REPORT_REPORTERUCODEID_LEN 5
#define CRASHCAT_REPORT_LIBOSVERSIONMAJOR_LEN 5
#define CRASHCAT_REPORT_LIBOSVERSIONMINOR_LEN 5
#define CRASHCAT_REPORT_LIBOSVERSIONCL_LEN 5
#define CRASHCAT_REPORT_TIMESTAMP_LEN 10
#define CRASHCAT_REPORT_CSRSTATE_LEN 91
#define CRASHCAT_REPORT_GPRSTATE_LEN 355
#define CRASHCAT_REPORT_STACKTRACE_LEN 10
#define CRASHCAT_REPORT_PCTRACE_LEN 10
#define CRASHCAT_REPORT_LOCALIOREGISTER_LEN 15
#define CRASHCAT_REPORT_BUILDID_LEN 1

extern const PRB_SERVICE_DESC prb_services_crashcat[];

// Service descriptor pointers

#endif // G_CRASHCAT_PB_H__
