/*
 * SPDX-FileCopyrightText: Copyright (c) 2016-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

/* THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT! */

#include "core/bin_data.h"
#include "rmflcnbl.h"
#include "nvBinSegment.h"
#include "../src/kernel/core/bin_data_pvt.h"

//
// Forward declarations.
// These are needed for generating the bindata access functions, which use
// "struct X" as a parameter. This is easier than #include-ing all the necessary
// headers. TODO: consider sharing a list of forward struct declarations with
// e.g. eng_desc.h?
//

struct KernelPmu;
struct KernelGsp;
struct KernelSec2;
struct KernelGsplite;
struct Spdm;
struct Lpsec;

//
// Default segment for bindata to put the data arrays to paged memory.
// This segment is never locked.
//
CONS_SEGMENT("PAGErGEN")

#if !defined(NVLOG_PARSING)

typedef enum
{
    BINDATA_INDEX_ZERO_UNUSED                                                                                      = 0,
    __idx_kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_IMAGE                                                  = 1,
    __idx_kgspBinArchiveGspRmBoot_TU102_BINDATA_LABEL_UCODE_DESC                                                   = 2,
    __idx_kgspBinArchiveGspRmBoot_GA100_BINDATA_LABEL_UCODE_IMAGE                                                  = 3,
    __idx_kgspBinArchiveGspRmBoot_GA100_BINDATA_LABEL_UCODE_DESC                                                   = 4,
    __idx_kgspBinArchiveGspRmBoot_GA102_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 5,
    __idx_kgspBinArchiveGspRmBoot_GA102_BINDATA_LABEL_UCODE_DESC_DBG                                               = 6,
    __idx_kgspBinArchiveGspRmBoot_GA102_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 7,
    __idx_kgspBinArchiveGspRmBoot_GA102_BINDATA_LABEL_UCODE_DESC_PROD                                              = 8,
    __idx_kgspBinArchiveGspRmBoot_AD102_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 9,
    __idx_kgspBinArchiveGspRmBoot_AD102_BINDATA_LABEL_UCODE_DESC_DBG                                               = 10,
    __idx_kgspBinArchiveGspRmBoot_AD102_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 11,
    __idx_kgspBinArchiveGspRmBoot_AD102_BINDATA_LABEL_UCODE_DESC_PROD                                              = 12,
    __idx_kgspBinArchiveGspRmBoot_GH100_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 13,
    __idx_kgspBinArchiveGspRmBoot_GH100_BINDATA_LABEL_UCODE_DESC_DBG                                               = 14,
    __idx_kgspBinArchiveGspRmBoot_GH100_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 15,
    __idx_kgspBinArchiveGspRmBoot_GH100_BINDATA_LABEL_UCODE_DESC_PROD                                              = 16,
    __idx_kgspBinArchiveGspRmBoot_GB100_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 17,
    __idx_kgspBinArchiveGspRmBoot_GB100_BINDATA_LABEL_UCODE_DESC_DBG                                               = 18,
    __idx_kgspBinArchiveGspRmBoot_GB100_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 19,
    __idx_kgspBinArchiveGspRmBoot_GB100_BINDATA_LABEL_UCODE_DESC_PROD                                              = 20,
    __idx_kgspBinArchiveGspRmBoot_GB10B_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 21,
    __idx_kgspBinArchiveGspRmBoot_GB10B_BINDATA_LABEL_UCODE_DESC_DBG                                               = 22,
    __idx_kgspBinArchiveGspRmBoot_GB10B_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 23,
    __idx_kgspBinArchiveGspRmBoot_GB10B_BINDATA_LABEL_UCODE_DESC_PROD                                              = 24,
    __idx_kgspBinArchiveGspRmBoot_GR100_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 25,
    __idx_kgspBinArchiveGspRmBoot_GR100_BINDATA_LABEL_UCODE_DESC_DBG                                               = 26,
    __idx_kgspBinArchiveGspRmBoot_GR100_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 27,
    __idx_kgspBinArchiveGspRmBoot_GR100_BINDATA_LABEL_UCODE_DESC_PROD                                              = 28,
    __idx_kgspBinArchiveGspRmBoot_GB202_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 29,
    __idx_kgspBinArchiveGspRmBoot_GB202_BINDATA_LABEL_UCODE_DESC_DBG                                               = 30,
    __idx_kgspBinArchiveGspRmBoot_GB202_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 31,
    __idx_kgspBinArchiveGspRmBoot_GB202_BINDATA_LABEL_UCODE_DESC_PROD                                              = 32,
    __idx_kgspBinArchiveGspRmBoot_GB20B_BINDATA_LABEL_UCODE_IMAGE_DBG                                              = 33,
    __idx_kgspBinArchiveGspRmBoot_GB20B_BINDATA_LABEL_UCODE_DESC_DBG                                               = 34,
    __idx_kgspBinArchiveGspRmBoot_GB20B_BINDATA_LABEL_UCODE_IMAGE_PROD                                             = 35,
    __idx_kgspBinArchiveGspRmBoot_GB20B_BINDATA_LABEL_UCODE_DESC_PROD                                              = 36,
    __idx_kgspBinArchiveConcatenatedFMCDesc_GH100_BINDATA_LABEL_UCODE_DESC_PROD                                    = 37,
    __idx_kgspBinArchiveConcatenatedFMCDesc_GB100_BINDATA_LABEL_UCODE_DESC_PROD                                    = 38,
    __idx_kgspBinArchiveConcatenatedFMCDesc_GB202_BINDATA_LABEL_UCODE_DESC_PROD                                    = 39,
    __idx_kgspBinArchiveConcatenatedFMCDesc_GR100_BINDATA_LABEL_UCODE_DESC_PROD                                    = 40,
    __idx_kgspBinArchiveConcatenatedFMC_GH100_BINDATA_LABEL_UCODE_IMAGE_PROD                                       = 41,
    __idx_kgspBinArchiveConcatenatedFMC_GB100_BINDATA_LABEL_UCODE_IMAGE_PROD                                       = 42,
    __idx_kgspBinArchiveConcatenatedFMC_GB202_BINDATA_LABEL_UCODE_IMAGE_PROD                                       = 43,
    __idx_kgspBinArchiveConcatenatedFMC_GR100_BINDATA_LABEL_UCODE_IMAGE_PROD                                       = 44,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GH100_BINDATA_LABEL_UCODE_IMAGE                                     = 45,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GH100_BINDATA_LABEL_UCODE_HASH                                      = 46,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GH100_BINDATA_LABEL_UCODE_SIG                                       = 47,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GH100_BINDATA_LABEL_UCODE_PKEY                                      = 48,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB100_BINDATA_LABEL_UCODE_IMAGE                                     = 49,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB100_BINDATA_LABEL_UCODE_HASH                                      = 50,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB100_BINDATA_LABEL_UCODE_SIG                                       = 51,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB100_BINDATA_LABEL_UCODE_PKEY                                      = 52,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB10B_BINDATA_LABEL_UCODE_IMAGE                                     = 53,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB10B_BINDATA_LABEL_UCODE_HASH                                      = 54,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB10B_BINDATA_LABEL_UCODE_SIG                                       = 55,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB10B_BINDATA_LABEL_UCODE_PKEY                                      = 56,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB202_BINDATA_LABEL_UCODE_IMAGE                                     = 57,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB202_BINDATA_LABEL_UCODE_HASH                                      = 58,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB202_BINDATA_LABEL_UCODE_SIG                                       = 59,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB202_BINDATA_LABEL_UCODE_PKEY                                      = 60,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB20B_BINDATA_LABEL_UCODE_IMAGE                                     = 61,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB20B_BINDATA_LABEL_UCODE_HASH                                      = 62,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB20B_BINDATA_LABEL_UCODE_SIG                                       = 63,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GB20B_BINDATA_LABEL_UCODE_PKEY                                      = 64,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GR100_BINDATA_LABEL_UCODE_IMAGE                                     = 65,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GR100_BINDATA_LABEL_UCODE_HASH                                      = 66,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GR100_BINDATA_LABEL_UCODE_SIG                                       = 67,
    __idx_kgspBinArchiveGspRmFmcGfwDebugSigned_GR100_BINDATA_LABEL_UCODE_PKEY                                      = 68,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_IMAGE                                      = 69,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_HASH                                       = 70,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_SIG                                        = 71,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_PKEY                                       = 72,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_IMAGE                                      = 73,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_HASH                                       = 74,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_SIG                                        = 75,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_PKEY                                       = 76,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB10B_BINDATA_LABEL_UCODE_IMAGE                                      = 77,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB10B_BINDATA_LABEL_UCODE_HASH                                       = 78,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB10B_BINDATA_LABEL_UCODE_SIG                                        = 79,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB10B_BINDATA_LABEL_UCODE_PKEY                                       = 80,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_IMAGE                                      = 81,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_HASH                                       = 82,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_SIG                                        = 83,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_PKEY                                       = 84,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB20B_BINDATA_LABEL_UCODE_IMAGE                                      = 85,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB20B_BINDATA_LABEL_UCODE_HASH                                       = 86,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB20B_BINDATA_LABEL_UCODE_SIG                                        = 87,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GB20B_BINDATA_LABEL_UCODE_PKEY                                       = 88,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_IMAGE                                      = 89,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_HASH                                       = 90,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_SIG                                        = 91,
    __idx_kgspBinArchiveGspRmFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_PKEY                                       = 92,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_IMAGE                                    = 93,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_HASH                                     = 94,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_SIG                                      = 95,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GH100_BINDATA_LABEL_UCODE_PKEY                                     = 96,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_IMAGE                                    = 97,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_HASH                                     = 98,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_SIG                                      = 99,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB100_BINDATA_LABEL_UCODE_PKEY                                     = 100,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_IMAGE                                    = 101,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_HASH                                     = 102,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_SIG                                      = 103,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GB202_BINDATA_LABEL_UCODE_PKEY                                     = 104,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_IMAGE                                    = 105,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_HASH                                     = 106,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_SIG                                      = 107,
    __idx_kgspBinArchiveGspRmCcFmcGfwProdSigned_GR100_BINDATA_LABEL_UCODE_PKEY                                     = 108,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_IMAGE_DBG                                              = 109,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_HEADER_DBG                                             = 110,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_IMAGE_PROD                                             = 111,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_HEADER_PROD                                            = 112,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_SIG_DBG                                                = 113,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_SIG_PROD                                               = 114,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_PATCH_LOC                                              = 115,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_PATCH_SIG                                              = 116,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_PATCH_META                                             = 117,
    __idx_kgspBinArchiveBooterLoadUcode_TU102_BINDATA_LABEL_NUM_SIGS                                               = 118,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_IMAGE_DBG                                              = 119,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_HEADER_DBG                                             = 120,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_IMAGE_PROD                                             = 121,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_HEADER_PROD                                            = 122,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_SIG_DBG                                                = 123,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_SIG_PROD                                               = 124,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_PATCH_LOC                                              = 125,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_PATCH_SIG                                              = 126,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_PATCH_META                                             = 127,
    __idx_kgspBinArchiveBooterLoadUcode_TU116_BINDATA_LABEL_NUM_SIGS                                               = 128,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_IMAGE_DBG                                              = 129,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_HEADER_DBG                                             = 130,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_IMAGE_PROD                                             = 131,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_HEADER_PROD                                            = 132,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_SIG_DBG                                                = 133,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_SIG_PROD                                               = 134,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_PATCH_LOC                                              = 135,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_PATCH_SIG                                              = 136,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_PATCH_META                                             = 137,
    __idx_kgspBinArchiveBooterLoadUcode_GA100_BINDATA_LABEL_NUM_SIGS                                               = 138,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_IMAGE_DBG                                              = 139,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_HEADER_DBG                                             = 140,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_IMAGE_PROD                                             = 141,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_HEADER_PROD                                            = 142,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_SIG_DBG                                                = 143,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_SIG_PROD                                               = 144,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_PATCH_LOC                                              = 145,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_PATCH_SIG                                              = 146,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_PATCH_META                                             = 147,
    __idx_kgspBinArchiveBooterLoadUcode_GA102_BINDATA_LABEL_NUM_SIGS                                               = 148,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_IMAGE_DBG                                              = 149,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_HEADER_DBG                                             = 150,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_IMAGE_PROD                                             = 151,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_HEADER_PROD                                            = 152,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_SIG_DBG                                                = 153,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_SIG_PROD                                               = 154,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_PATCH_LOC                                              = 155,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_PATCH_SIG                                              = 156,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_PATCH_META                                             = 157,
    __idx_kgspBinArchiveBooterLoadUcode_AD102_BINDATA_LABEL_NUM_SIGS                                               = 158,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_IMAGE_DBG                                            = 159,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_HEADER_DBG                                           = 160,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_IMAGE_PROD                                           = 161,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_HEADER_PROD                                          = 162,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_SIG_DBG                                              = 163,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_SIG_PROD                                             = 164,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_PATCH_LOC                                            = 165,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_PATCH_SIG                                            = 166,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_PATCH_META                                           = 167,
    __idx_kgspBinArchiveBooterUnloadUcode_TU102_BINDATA_LABEL_NUM_SIGS                                             = 168,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_IMAGE_DBG                                            = 169,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_HEADER_DBG                                           = 170,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_IMAGE_PROD                                           = 171,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_HEADER_PROD                                          = 172,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_SIG_DBG                                              = 173,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_SIG_PROD                                             = 174,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_PATCH_LOC                                            = 175,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_PATCH_SIG                                            = 176,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_PATCH_META                                           = 177,
    __idx_kgspBinArchiveBooterUnloadUcode_TU116_BINDATA_LABEL_NUM_SIGS                                             = 178,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_IMAGE_DBG                                            = 179,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_HEADER_DBG                                           = 180,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_IMAGE_PROD                                           = 181,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_HEADER_PROD                                          = 182,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_SIG_DBG                                              = 183,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_SIG_PROD                                             = 184,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_PATCH_LOC                                            = 185,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_PATCH_SIG                                            = 186,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_PATCH_META                                           = 187,
    __idx_kgspBinArchiveBooterUnloadUcode_GA100_BINDATA_LABEL_NUM_SIGS                                             = 188,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_IMAGE_DBG                                            = 189,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_HEADER_DBG                                           = 190,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_IMAGE_PROD                                           = 191,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_HEADER_PROD                                          = 192,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_SIG_DBG                                              = 193,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_SIG_PROD                                             = 194,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_PATCH_LOC                                            = 195,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_PATCH_SIG                                            = 196,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_PATCH_META                                           = 197,
    __idx_kgspBinArchiveBooterUnloadUcode_GA102_BINDATA_LABEL_NUM_SIGS                                             = 198,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_IMAGE_DBG                                            = 199,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_HEADER_DBG                                           = 200,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_IMAGE_PROD                                           = 201,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_HEADER_PROD                                          = 202,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_SIG_DBG                                              = 203,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_SIG_PROD                                             = 204,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_PATCH_LOC                                            = 205,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_PATCH_SIG                                            = 206,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_PATCH_META                                           = 207,
    __idx_kgspBinArchiveBooterUnloadUcode_AD102_BINDATA_LABEL_NUM_SIGS                                             = 208,
    __idx_ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_IMAGE                                                   = 209,
    __idx_ksec2BinArchiveBlUcode_TU102_BINDATA_LABEL_UCODE_DESC                                                    = 210,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_IMAGE_DBG                                            = 211,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_HEADER_DBG                                           = 212,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_IMAGE_PROD                                           = 213,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_HEADER_PROD                                          = 214,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_SIG_DBG                                              = 215,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_SIG_PROD                                             = 216,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_PATCH_LOC                                            = 217,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_PATCH_SIG                                            = 218,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_PATCH_META                                           = 219,
    __idx_ksec2BinArchiveSecurescrubUcode_AD10X_BINDATA_LABEL_NUM_SIGS                                             = 220,
    __idx_spdmBinArchiveL1Certificate_IMPL_BINDATA_LABEL_CERTIFICATE_PEM                                           = 221,
    __idx_spdmBinArchiveIndividualL2Certificate_GH100_BINDATA_LABEL_CERTIFICATE_PEM                                = 222,
    __idx_spdmBinArchiveIndividualL3Certificate_GH100_BINDATA_LABEL_CERTIFICATE_PEM                                = 223,
    __idx_spdmBinArchiveIndividualL2Certificate_GB100_BINDATA_LABEL_CERTIFICATE_PEM                                = 224,
    __idx_spdmBinArchiveIndividualL3Certificate_GB100_BINDATA_LABEL_CERTIFICATE_PEM                                = 225,
    __idx_spdmBinArchiveIndividualL3Certificate_GB102_BINDATA_LABEL_CERTIFICATE_PEM                                = 226,
    __idx_spdmBinArchiveIndividualL3Certificate_GB110_BINDATA_LABEL_CERTIFICATE_PEM                                = 227,
    __idx_spdmBinArchiveIndividualL3Certificate_GB112_BINDATA_LABEL_CERTIFICATE_PEM                                = 228,
    __idx_spdmBinArchiveIndividualL2Certificate_GB20X_BINDATA_LABEL_CERTIFICATE_PEM                                = 229,
    __idx_spdmBinArchiveIndividualL3Certificate_GB202_BINDATA_LABEL_CERTIFICATE_PEM                                = 230,
    __idx_spdmBinArchiveIndividualL3Certificate_GB203_BINDATA_LABEL_CERTIFICATE_PEM                                = 231,
    __idx_spdmBinArchiveIndividualL3Certificate_GB205_BINDATA_LABEL_CERTIFICATE_PEM                                = 232,
    __idx_spdmBinArchiveIndividualL3Certificate_GB206_BINDATA_LABEL_CERTIFICATE_PEM                                = 233,
    __idx_spdmBinArchiveIndividualL3Certificate_GB207_BINDATA_LABEL_CERTIFICATE_PEM                                = 234,
    __idx_spdmBinArchiveIndividualL2Certificate_GR1XX_BINDATA_LABEL_CERTIFICATE_PEM                                = 235,
    __idx_spdmBinArchiveIndividualL3Certificate_GR100_BINDATA_LABEL_CERTIFICATE_PEM                                = 236,
    __idx_spdmBinArchiveIndividualL3Certificate_GR102_BINDATA_LABEL_CERTIFICATE_PEM                                = 237,

    BINDATA_INDEX_END
} bindata_index_t;

//
// The fingerprint is a hash of all the possible bindata_index_t values.
// It changes if any values are added, removed or reshuffled.
//
const NvU64 g_bindata_fingerprint = 0xee84afffcb8c7f6dull;


struct BINDATA_STORAGE_PVT_ALL
{
#define BINDATA_NO_SEGMENTS
#define BINDATA_INCLUDE_STORAGE_PVT_DECL

#include "g_bindata_kgspGetBinArchiveGspRmBoot_TU102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_AD102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB20B.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GR100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_AD102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_AD102.c"
#include "g_bindata_ksec2GetBinArchiveBlUcode_TU102.c"
#include "g_bindata_ksec2GetBinArchiveSecurescrubUcode_AD10X.c"
#include "g_bindata_spdmGetBinArchiveL1Certificate_IMPL.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB102.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB110.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB112.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB20X.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB202.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB203.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB205.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB206.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB207.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GR1XX.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR102.c"

#undef BINDATA_INCLUDE_STORAGE_PVT_DECL
#undef BINDATA_NO_SEGMENTS
};

// First include the data as it's referenced by storage_pvt
// Also useful to keep the NvU8 data packed together so we can merge multiple
// buffers into a single chunk for allocations.
#define BINDATA_INCLUDE_DATA

#include "g_bindata_kgspGetBinArchiveGspRmBoot_TU102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_AD102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB20B.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GR100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_AD102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_AD102.c"
#include "g_bindata_ksec2GetBinArchiveBlUcode_TU102.c"
#include "g_bindata_ksec2GetBinArchiveSecurescrubUcode_AD10X.c"
#include "g_bindata_spdmGetBinArchiveL1Certificate_IMPL.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB102.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB110.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB112.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB20X.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB202.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB203.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB205.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB206.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB207.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GR1XX.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR102.c"

#undef BINDATA_INCLUDE_DATA

BINDATA_CONST BINDATA_STORAGE_PVT g_bindata_pvt[] =
{
#define BINDATA_NO_SEGMENTS
#define BINDATA_INCLUDE_STORAGE_PVT_DEFN
// Add a dummy entry at index 0 which will be mapped to BINDATA_INDEX_ZERO_UNUSED in bindata_index_t
{0},

#include "g_bindata_kgspGetBinArchiveGspRmBoot_TU102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_AD102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB20B.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GR100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_AD102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_AD102.c"
#include "g_bindata_ksec2GetBinArchiveBlUcode_TU102.c"
#include "g_bindata_ksec2GetBinArchiveSecurescrubUcode_AD10X.c"
#include "g_bindata_spdmGetBinArchiveL1Certificate_IMPL.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB102.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB110.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB112.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB20X.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB202.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB203.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB205.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB206.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB207.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GR1XX.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR102.c"

#undef BINDATA_INCLUDE_STORAGE_PVT_DEFN
#undef BINDATA_NO_SEGMENTS
};

const NvU32 g_bindata_pvt_count = sizeof(g_bindata_pvt) / sizeof(BINDATA_STORAGE_PVT);

// Lastly, include everything else..
#define BINDATA_INCLUDE_ARCHIVE
#define BINDATA_INCLUDE_FUNCTION

#include "g_bindata_kgspGetBinArchiveGspRmBoot_TU102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GA102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_AD102.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmBoot_GB20B.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMCDesc_GR100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GH100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB100.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GB202.c"
#include "g_bindata_kgspGetBinArchiveConcatenatedFMC_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwDebugSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB10B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GB20B.c"
#include "g_bindata_kgspGetBinArchiveGspRmFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GH100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB100.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GB202.c"
#include "g_bindata_kgspGetBinArchiveGspRmCcFmcGfwProdSigned_GR100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterLoadUcode_AD102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_TU116.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA100.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_GA102.c"
#include "g_bindata_kgspGetBinArchiveBooterUnloadUcode_AD102.c"
#include "g_bindata_ksec2GetBinArchiveBlUcode_TU102.c"
#include "g_bindata_ksec2GetBinArchiveSecurescrubUcode_AD10X.c"
#include "g_bindata_spdmGetBinArchiveL1Certificate_IMPL.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GH100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB102.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB110.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB112.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GB20X.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB202.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB203.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB205.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB206.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GB207.c"
#include "g_bindata_spdmGetBinArchiveIndividualL2Certificate_GR1XX.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR100.c"
#include "g_bindata_spdmGetBinArchiveIndividualL3Certificate_GR102.c"

#undef BINDATA_INCLUDE_FUNCTION
#undef BINDATA_INCLUDE_ARCHIVE

#endif // !defined(NVLOG_PARSING)
