// This file is automatically generated by rmconfig - DO NOT EDIT!
//
// NVOC Header State : This file is used for different code path for disabled NVH
//
// Profile:  shipping-gpus-openrm
// Template: templates/gt_nvh_state.h
//

#ifndef _G_NVH_STATE_H_
#define _G_NVH_STATE_H_

//
// __nvoc_nvh_state_guard
//  This macro define is used to check whether this header is included before
//  NVOC headers.  The usage:
//    #ifndef __nvoc_nvh_state_guard
//    #error "NVH state guard header is not included prior to this NVOC header"
//    #endif
//
#define __nvoc_nvh_state_guard

//
// List of disabled NVOC headers
//



#endif // _G_NVH_STATE_H_
