/*
 * SPDX-FileCopyrightText: Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef ad102_dev_nv_bus_addendum_h
#define ad102_dev_nv_bus_addendum_h

// Scratch bits for Global EROT Grant scratch bits
#define NV_PBUS_SW_GLOBAL_EROT_GRANT                                NV_PBUS_SW_SCRATCH(0)
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_VALID                          3:3
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_VALID_YES                      1
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_VALID_NO                       0
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_REQUEST                        4:4
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_REQUEST_SET                    1
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_REQUEST_CLEAR                  0
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_ALLOW                          5:5
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_ALLOW_YES                      1
#define NV_PBUS_SW_GLOBAL_EROT_GRANT_ALLOW_NO                       0

#endif // ad102_dev_nv_bus_addendum_h

